
/*
KnotJob V3.24

Copyright (C) 2015 Dirk Schuetz <dirk.schuetz@durham.ac.uk> and Lewin Strauss

    This file is part of KnotJob.

    KnotJob is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    KnotJob is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;

public class KnotJob {
    
    ArrayList<diagram> sockflow = new ArrayList<diagram>();
    ArrayList<int[]> jones; // required for findJonesGroupings
    
    public KnotJob(String [] args) {
        link Link = null;
        boolean mirror = false;
        int next = 0;
        int cas = -1;
        int hdeg = 0;
        int qdeg = 0;
        int n = 0;
        int fool = 0;
        while ((next < args.length) & (fool == 0)) {
            if (args[next].equals("-a")) {
                try {
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 0;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-h")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 1;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-s1")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 2;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-s2")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 3;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-s3")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 4;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-s21")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 5;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-s22")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 6;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-st")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 7;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-h4")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 8;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-b24")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 9;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-b42")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 10;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-b44")) {
                try {
                    next++;
                    hdeg = Integer.parseInt(args[next]);
                    next++;
                    qdeg = Integer.parseInt(args[next]);
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 11;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-a4")) {
                try {
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 12;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-j")) {
                try {
                    next++;
                    n = Integer.parseInt(args[next]);
                    next++;
                    cas = 14;
                }
                catch (Exception e) {
                    fool = 2;
                }
            }
            else if (args[next].equals("-m")) {
                next++;
                mirror = true;
            }
            else if (args[next].equals("-dg")) {
                next++;
                Link = doubleGauss(args[next]);
                next++;
            }
            else if (args[next].equals("-n")) {
                next++;
                Link = diagramCreator(args[next],args[next+1]);
                next++;
                next++;
            }
            else fool = 3;
        }
        
        if (fool > 0) {
            switch (fool) {
                case 1 : System.err.println("Wrong argument at entry "+ next+".");
                break;
                case 2 : System.err.println("Unknown command.");
                break;
                case 3 : System.err.println("Not enough arguments.");
                break;
                case 4 : System.err.println("Wrong crossings.");
                break;
                case 5 : System.err.println("Wrong paths.");
                break;
            }
            return;
        }
        
        if (mirror) Link = Mirror(Link);
        
        if (n>2) Link = Mirror(Link);
        ObtainSock(Link);
        
        switch (cas) {
            case 0 : AllMod2Hom(Link,n);
            break;
            case 1 : Mod2Homology(Link,hdeg,qdeg,n);
            break;
            case 2 : SquareOne(Link,hdeg,qdeg,n);
            break;
            case 3 : SquareTwo(Link,hdeg,qdeg,n);
            break;
            case 4 : SquareThree(Link,hdeg,qdeg,n);
            break;
            case 5 : SquareTwoOne(Link,hdeg,qdeg,n);
            break;
            case 6 : SquareTwoTwo(Link,hdeg,qdeg,n);
            break;
            case 7 : SteenSq(Link,hdeg,qdeg,n);
            break;
            case 8 : calcMod4Homology(Link,hdeg,qdeg,n,true);
            break;
            case 9 : BocksteinTwoFour(Link,hdeg,qdeg,n);
            break;
            case 10 : BocksteinFourTwo(Link,hdeg,qdeg,n);
            break;
            case 11 : BocksteinFourFour(Link,hdeg,qdeg,n);
            break;
            case 12 : calcAllMod4Homology(Link,n);
            break;
            case 14 : calcJones(Link,n);
            break;
        }
    }
  
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    /** 
     * Calculates the Bockstein: KH^(hdeg,q)(Z/2) --> KH^(hdeg+1,qdeg)(Z/4) of a linkdiagram.
     */
    final void BocksteinTwoFour(link Link,int hdeg,int qdeg,int n) {
        ArrayList<flow_object> objects0; 
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        ArrayList<m4obj> listobj00 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj11 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj22 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj33 = new ArrayList<m4obj>(); 
        
        objects0 = getObjects(Link, hdeg - 1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg + 1, qdeg, n);
        objects3 = getObjects(Link, hdeg + 2, qdeg, n);
        get0Mod(objects1, objects0, Link.paths, n);
        get0Mod(objects2, objects1, Link.paths, n);
        get0Mod(objects3, objects2, Link.paths, n);
        
        resetChainCx_2(objects0, listobj0, objects1, listobj1, objects2, listobj2);
        ArrayList<ArrayList<Integer>> cocyclesBotBool = getCocycles(listobj0, listobj1, listobj2);
        ArrayList<Cocycle> cocyclesBot = convertCocycles(cocyclesBotBool);
        
        for (Cocycle gener : cocyclesBot) {
            System.out.print("Cocycle (mod2): ");
            gener.print(false);
        }
        
        resetChainCx_0(objects1, listobj11, objects2, listobj22, objects3, listobj33); 
        ArrayList<Cocycle> cocyclesBock = Bockstein_n_m(cocyclesBot, listobj11, listobj22, 2, 4);
        
        for (Cocycle gener : cocyclesBock) {
            System.out.print("Bockstein (mod4): ");
            gener.print(true);
        }
        
        for (Cocycle linComb : cocyclesBock) { 
            resetChainCx_4(objects1, listobj11, objects2,listobj22, objects3, listobj33);
            linComb = DetHomCls_4(linComb, listobj11, listobj22, listobj33);
        }
        
        
        for (Cocycle gener : cocyclesBock) {
            System.out.print("Represents (mod4): ");
            gener.print(true);
        }
    }
    
    /** 
     * Calculates the Bockstein: KH^(hdeg,q)(Z/4) --> KH^(hdeg+1,qdeg)(Z/2) of a linkdiagram.
     */
    final void BocksteinFourTwo(link Link,int hdeg,int qdeg,int n) {
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj3 = new ArrayList<m2obj>();
        ArrayList<m4obj> listobj00 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj11 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj22 = new ArrayList<m4obj>(); 
        
        objects0 = getObjects(Link, hdeg - 1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg + 1, qdeg, n);
        objects3 = getObjects(Link, hdeg + 2, qdeg, n);
        get0Mod(objects1, objects0, Link.paths, n);
        get0Mod(objects2, objects1, Link.paths, n);
        get0Mod(objects3, objects2, Link.paths, n);
        
        resetChainCx_4(objects0, listobj00, objects1, listobj11, objects2, listobj22); 
        ArrayList<Cocycle> cocyclesBot = getCocycles_4(listobj00, listobj11, listobj22);
       
        for (Cocycle gener : cocyclesBot) {
            System.out.print("Cocycle (mod4): ");
            gener.print(true);
        }
        
        resetChainCx_0(objects0, listobj00, objects1, listobj11, objects2, listobj22); 
        ArrayList<Cocycle> cocyclesBock = Bockstein_n_m(cocyclesBot, listobj11, listobj22, 4, 2);
        
        for (Cocycle gener : cocyclesBock) {
            System.out.print("Bockstein (mod2): ");
            gener.print(false);
        }
        
        ArrayList<ArrayList<Integer>> cocyclesBockBool = convertBackCocycles(cocyclesBock);
        resetChainCx_2(objects1, listobj1, objects2, listobj2, objects3, listobj3); 
        DetHomCls(cocyclesBockBool, listobj1, listobj2, listobj3);
        cocyclesBock = convertCocycles(cocyclesBockBool);

        for (Cocycle gener : cocyclesBock) {
            System.out.print("Represents (mod2): ");
            gener.print(false);
        }   
    } 

    /** 
     * Calculates the Bockstein: KH^(hdeg,q)(Z/4) --> KH^(hdeg+1,qdeg)(Z/4) of a linkdiagram.
     */
    final void BocksteinFourFour(link Link,int hdeg,int qdeg,int n) {
        ArrayList<flow_object> objects0; 
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<m4obj> listobj0 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj1 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj2 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj3 = new ArrayList<m4obj>(); 
        
        objects0 = getObjects(Link, hdeg - 1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg + 1, qdeg, n);
        objects3 = getObjects(Link, hdeg + 2, qdeg, n);
        get0Mod(objects1, objects0, Link.paths, n);
        get0Mod(objects2, objects1, Link.paths, n);
        get0Mod(objects3, objects2, Link.paths, n);

        resetChainCx_4(objects0, listobj0, objects1, listobj1, objects2, listobj2); 
        ArrayList<Cocycle> cocyclesBot = getCocycles_4(listobj0, listobj1, listobj2);
       
        for (Cocycle gener : cocyclesBot) {
            System.out.print("Cocycle (mod4): ");
            gener.print(true);
        }
        
        resetChainCx_0(objects1, listobj1, objects2, listobj2, objects3, listobj3); 
        ArrayList<Cocycle> cocyclesBock = Bockstein_n_m(cocyclesBot, listobj1, listobj2, 4, 4);
        
        for (Cocycle gener : cocyclesBock) {
            System.out.print("Bockstein (mod4): ");
            gener.print(true);
        }
        
        for (Cocycle linComb : cocyclesBock) { 
            resetChainCx_4(objects1, listobj1, objects2,listobj2, objects3, listobj3);
            linComb = DetHomCls_4(linComb, listobj1, listobj2, listobj3);
        }
        for (Cocycle gener : cocyclesBock) {
            System.out.print("Represents (mod4): ");
            gener.print(true);
        }
    }
    
    boolean first;
    
    /**
     * Calculates all mod4 Khovanov cohomology groups of a linkdiagram.
     */
    final void calcAllMod4Homology(link Link, int n) {
        int qmin = minQ(Link, n);
        int qmax = maxQ(Link, n);
        int hmin = minH(Link, n);
        int hmax = maxH(Link, n);
        
        for (int q = qmax; q >= qmin; q = q - 2) {
            first = true;
            for (int h = hmin; h <= hmax; h++) calcMod4Homology(Link, h, q, n, false);
            if (!first) System.out.println(" ) q^"+q);
        }
    } 
    
    /**
     * Determines the Jones Polynomial from linkdiagram infromation.
     */
    final void calcJones(link Link, int n) {
        int qmin = minQ(Link, n);
        int qmax = maxQ(Link, n);
        int hmin = minH(Link, n);
        int hmax = maxH(Link, n);
        ArrayList<int[]> poly = new ArrayList<int[]>();
        
        for (int q = qmin; q <= qmax; q = q + 2) {
            int factor = 0;
            for (int h = hmin; h <= hmax; h++) { 
                ArrayList<flow_object> objects = getObjects(Link, h, q, n); 
                if (h % 2 == 0) factor += objects.size();
                else factor -= objects.size();
            }
            if (factor != 0) {
                int[] summand = new int[2];
                summand[0] = factor;
                summand[1] = q;
                poly.add(summand);
            }
        }
        this.jones = poly;
        
        System.out.print("Jones: ");
        for (int i = 0; i < poly.size(); i ++) {
            if (i != poly.size() - 1) System.out.print("(" + poly.get(i)[0] + ")" + " q^" + "(" +poly.get(i)[1] + ")" + " + ");
            else System.out.print("(" + poly.get(i)[0] + ")" + " q^" + "(" +poly.get(i)[1] + ")");
        }
        System.out.println();
        
    } 

    /**
     * Calculates the mod4 Khovanov cohomology of a linkdiagram
     * for given homological and quantum degree.
     */
    void calcMod4Homology(link Link, int hdeg, int qdeg, int n, boolean show) {
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2; 
        ArrayList<m4obj> listobj0 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj1 = new ArrayList<m4obj>();
        ArrayList<m4obj> listobj2 = new ArrayList<m4obj>();
        
        objects2 = getObjects(Link, hdeg + 1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n); 
        objects0 = getObjects(Link, hdeg - 1, qdeg, n);
        get0Mod(objects1, objects0, Link.paths,n);
        get0Mod(objects2, objects1, Link.paths, n);
        
        resetChainCx_4(objects0, listobj0, objects1, listobj1, objects2, listobj2);
        simplifyChainCx_4(listobj0, listobj1, listobj2);
        int[][] left = detBoundMat(listobj0, listobj1); 
        int[][] right = detBoundMat(listobj1, listobj2);
        int[] homology = detHom_4(left, right, listobj1.size(), listobj2.size());
        
        if ((!show) & (homology[0] == 0) & (homology[1] == 0)) return;
        if (show) {
         System.out.print("sl("+n+") ");
         System.out.println("H("+hdeg+","+qdeg+")");
         System.out.println("Z/4-Summands : "+homology[0]);
         System.out.println("Z/2-Summands : "+homology[1]);
        }
        else {
            if (first) System.out.print("( ");
            else System.out.print(" + ");
            //if (homology[0] > 0 & homology[1] > 0) System.out.print("(");
            if (homology[1] > 0) {
                if (homology[0] > 0) {
                    System.out.print("("+homology[0]+" + ");
                    if (homology[1] > 1) System.out.print(homology[1]+"[2]) ");
                    else System.out.print("[2])");
                }
                else {
                    if (homology[1] > 1) System.out.print(homology[1]);
                    System.out.print("[2] ");
                }
                System.out.print("t^"+hdeg);
                first = false;
            }
            else {
                if (homology[0] > 0) {
                    if (homology[0] > 1) System.out.print(homology[0]+" ");
                    System.out.print("t^"+hdeg);
                    first = false;
                }
            }
        }
    }
    
    /**
     * Determines mod4 homology from boundary matrix information.
     */
    int[] detHom_4(int[][] left, int[][] right, int dim1, int dim2) {
        int dim = left.length;
        int dimR = right.length;
        int dimL = left[0].length;
        
        ArrayList<int[][]> diagR = IntegerMatrices.trackedDiag_4(right, true);
        right = diagR.get(0);
        int[][] colOpInv = diagR.get(1);
        left = IntegerMatrices.mult_n(colOpInv, left, 4); 
        
        int cutTop = 0;
        for (int i = (BasicMaths.min(dim, dimR) - 1); i >= 0; i--) {
            if (right[i][i] == 2) cutTop = i;
            if (right[i][i] == 1) break; 
        }
        int cutBot = BasicMaths.min(dim, dimR) - 1;
        for (int i = (BasicMaths.min(dim, dimR) - 1); i >= 0; i--) {
            if (right[i][i] == 0) cutBot = i;
            if (right[i][i] != 0) break;
        }
        int[][] leftNew = new int[dim][dimL];
        for (int i= 0; i< dim; i++) {
            leftNew[i] = left[BasicMaths.mod_n((cutBot + i), dim)];
        }
        left = IntegerMatrices.cutDiag(leftNew, cutTop);
        
        int[][] im = new int[1][dim];                             
        for (int i = 0; i < BasicMaths.min(dimL, dim); i++) {
            if (left[i][i] == 1) im[0][i] = 4; 
            else if (left[i][i] == 2) im[0][i] = 2;
            else if (left[i][i] == 0) break;
        }
        int[][] ker = new int[1][dim];                               
        if (dim1 != 0) {
            for (int i = 0; i < dim; i++) ker[0][i] = 4; 
            for (int i = 0; i < BasicMaths.min(dimR, dim); i++) {
                if (right[i][i] == 0) break;
                else if (right[i][i] == 2) ker[0][dim - 1 - i] = 2;
                else if (right[i][i] == 1) ker[0][dim - 1 - i] = 0;
            }
        }
        int[][] homology = IntegerMatrices.add_n(1, ker, -1, im, 10);   
        
        int fours = 0;
        int twos = 0;     
        for (int i = 0; i < homology[0].length; i++) {
            if (homology[0][i] == 4) fours += 1;
            else if (homology[0][i] == 2) twos += 1;
        }
        int[] result = {fours, twos};
        
        return result;                                                   
    }
    
    /**
     * Calculates the Bockstein for given cocycles 
     * from chaincomplex information.
     */
    ArrayList<Cocycle> Bockstein_n_m(ArrayList<Cocycle> cocyclesBot,
                                     ArrayList<m4obj> listobj1,
                                     ArrayList<m4obj> listobj2,
                                     int modBot,int modTop) {
        ArrayList<Cocycle> cocyclesTop = new ArrayList<Cocycle>();
        
        for (Cocycle cocy : cocyclesBot) { 
            Cocycle imCocy = new Cocycle(); 
            int index = 0;   

            for (int position : cocy.pos) { 
                int factor = cocy.factor.get(index);
                index ++;
                m4obj generatorBot = listobj1.get(position);
                for (Extm4obj generatorTopIt : generatorBot.above) { 
                    int intermEntry = factor*generatorTopIt.count;
                    if (! imCocy.pos.contains(generatorTopIt.obj.pos)) {
                        imCocy.pos.add(generatorTopIt.obj.pos);
                        imCocy.factor.add(intermEntry);
                    } else if (imCocy.factor.get(imCocy.pos.indexOf(generatorTopIt.obj.pos))
                                 + intermEntry == 0) { 
                        imCocy.factor.remove(imCocy.pos.indexOf(generatorTopIt.obj.pos));
                        imCocy.pos.remove((Integer) generatorTopIt.obj.pos);
                    } else imCocy.factor.set(imCocy.pos.indexOf(generatorTopIt.obj.pos), 
                                          intermEntry+ imCocy.factor.get(imCocy.pos.indexOf(generatorTopIt.obj.pos)));
                } 
            }
            
            for (Iterator<Integer> posiIt = imCocy.pos.iterator(); posiIt.hasNext();) {
                int posi = posiIt.next();
                int helpIndex = imCocy.pos.indexOf(posi);
                int entry = imCocy.factor.get(helpIndex);
                entry = BasicMaths.mod_n(entry / modBot, modTop);
                if (entry == 0) { 
                    posiIt.remove();
                    imCocy.factor.remove(helpIndex);
                } else imCocy.factor.set(helpIndex, entry);   
            }
            cocyclesTop.add(imCocy);  
        } 
        return cocyclesTop;  
    }
    
    
    /**
     * Determines the homology class of a given cocycle. (still messy)
     */
    Cocycle DetHomCls_4(Cocycle linComb, ArrayList<m4obj> listobj0,
                        ArrayList<m4obj> listobj1, ArrayList<m4obj> listobj2) { 
        Extm4obj y,w;
        m4obj cand;
        ArrayList<Integer> cancelList = new ArrayList<Integer>();
        
        Iterator<m4obj> candIt = listobj1.iterator();
        while (candIt.hasNext()) { 
            cand = candIt.next();
            if (cand.above.isEmpty() && cand.below.isEmpty()) {
                continue; 
            } else { 
                boolean found1or3 = false;
                boolean below = false;
                boolean above = false;
                y = null;
                
                // Find 1's or 3's
                if (cand.below.size() >= cand.above.size()) { 
                    Iterator<Extm4obj> yBelowIt = cand.below.iterator();
                    while (yBelowIt.hasNext()) {        
                        y = yBelowIt.next();
                        if (y.count != 2) {
                            found1or3 = true;
                            below = true;
                            break;
                        }
                    } if (found1or3 == false) {
                        Iterator<Extm4obj> yAboveIt = cand.above.iterator();
                        while (yAboveIt.hasNext()) {
                            y = yAboveIt.next();
                            if (y.count != 2) {
                                found1or3 = true;
                                above = true;
                                break;
                            }
                        }
                    }
                } else {
                    Iterator<Extm4obj> yAboveIt = cand.above.iterator();
                    while (yAboveIt.hasNext()) {
                        y = yAboveIt.next();
                        if (y.count != 2) {
                            found1or3 = true;
                            above = true;
                            break;
                        }
                    } if (found1or3 == false) {
                        Iterator<Extm4obj> yBelowIt = cand.below.iterator();
                        while (yBelowIt.hasNext()) {        
                            y = yBelowIt.next();
                            if (y.count != 2) {
                                found1or3 = true;
                                below = true;
                                break;
                            }
                        }
                    }
                }   
                if (found1or3 == false) continue;
                
                if (above) {
                    if (linComb.pos.contains(cand.pos)) {
                        linComb.factor.remove(linComb.pos.indexOf(cand.pos));
                        linComb.pos.remove((Integer) cand.pos);
                        cancelList.add(cand.pos);
                    }
                    Iterator<Extm4obj> wIt = cand.above.iterator();
                    while (wIt.hasNext()) { 
                        w = wIt.next();
                        if (w.obj == y.obj) continue;
                        for (Iterator<Extm4obj> bIt = y.obj.below.iterator(); bIt.hasNext();) { 
                            Extm4obj b = bIt.next();
                            int loopSigns = BasicMaths.mod_n(w.count * y.count * b.count, 4);
                            if (loopSigns==0) continue;
                            if (b.obj != cand) { 
                                Extm4obj wForB = null;
                                Iterator<Extm4obj> wForBIt = b.obj.above.iterator(); 
                                while (wForBIt.hasNext()) { 
                                    wForB = wForBIt.next();
                                    if (wForB.obj == w.obj) break;
                                }
                                if (wForB.obj == w.obj) { 
                                    Extm4obj bForW = null;
                                    Iterator<Extm4obj> bForWIt = w.obj.below.iterator(); 
                                    while (bForWIt.hasNext()) { 
                                        bForW = bForWIt.next();
                                        if (bForW.obj == b.obj) break;
                                    }
                                    int entry = BasicMaths.mod_n(wForB.count - loopSigns, 4); 
                                    if ( entry==0 ) { 
                                        wForBIt.remove(); 
                                        bForWIt.remove();
                                    } else { 
                                        wForB.count = entry;
                                        bForW.count = entry;
                                    }
                                } else { 
                                    Extm4obj bW = new Extm4obj (b.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    Extm4obj wB = new Extm4obj (w.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    w.obj.below.add(bW); 
                                    b.obj.above.add(wB);
                                }      
                            }   
                        } 
                    } 
                    for (Iterator<Extm4obj> zIt = y.obj.below.iterator(); zIt.hasNext();) { 
                        Extm4obj z = zIt.next();
                        for (Iterator<Extm4obj> yForZIt = z.obj.above.iterator(); yForZIt.hasNext();) {
                            Extm4obj yForZ = yForZIt.next();
                            if (yForZ.obj == y.obj) {
                                yForZIt.remove(); 
                                break;
                            }
                        }      
                    }
                    for (Iterator<m4obj> yyIt = listobj2.iterator(); yyIt.hasNext();) { 
                        m4obj yy = yyIt.next();
                        if (yy == y.obj) {
                            yyIt.remove(); 
                            break;
                        }
                    }
                } 
                
                else { 
                    if (linComb.pos.contains(cand.pos)) {
                        int candFactor = linComb.factor.get(linComb.pos.indexOf(cand.pos));
                        linComb.factor.remove(linComb.pos.indexOf(cand.pos));
                        linComb.pos.remove((Integer) cand.pos);
                        cancelList.add(cand.pos);
                        Iterator<Extm4obj> yAboveIter = y.obj.above.iterator();
                        while (yAboveIter.hasNext()) {
                            Extm4obj yAboveGen = yAboveIter.next();
                            if (cancelList.contains(yAboveGen.obj.pos)) continue;
                            if (linComb.pos.contains(yAboveGen.obj.pos)) {
                                int helperIndex = linComb.pos.indexOf(yAboveGen.obj.pos);
                                int factor = linComb.factor.get(helperIndex);
                                factor += (-1 * yAboveGen.count * y.count) * candFactor;
                                factor = BasicMaths.mod_n(factor, 4);
                                if (factor == 0) {
                                    linComb.factor.remove(helperIndex);
                                    linComb.pos.remove(helperIndex);
                                } else {
                                    linComb.factor.set(helperIndex, factor);
                                }
                            } else {
                                int factor = (-1 * yAboveGen.count * y.count) * candFactor;
                                factor = BasicMaths.mod_n(factor, 4);
                                if (factor != 0) {
                                    linComb.pos.add(yAboveGen.obj.pos);
                                    linComb.factor.add(factor);
                                }
                            }
                        }
                    }
                                       
                    Iterator<Extm4obj> wIt = cand.below.iterator();
                    while (wIt.hasNext()) {
                        w = wIt.next();
                        if (w.obj == y.obj) continue;
                        for (Iterator<Extm4obj> bIt = y.obj.above.iterator(); bIt.hasNext();) { 
                            Extm4obj b = bIt.next();
                            int loopSigns = BasicMaths.mod_n(w.count*y.count*b.count,4);
                            if (loopSigns==0) continue;
                            if (b.obj != cand) { // Change the matrix entries for w-->b
                                Extm4obj wForB = null;
                                Iterator<Extm4obj> wForBIt = b.obj.below.iterator(); // check whether and where w is in b.obj.below
                                while (wForBIt.hasNext()) { 
                                    wForB = wForBIt.next();
                                    if (wForB.obj == w.obj) break;
                                }
                                if (wForB.obj == w.obj) { // w already connected to b
                                    Extm4obj bForW = null; 
                                    Iterator<Extm4obj> bForWIt = w.obj.above.iterator(); 
                                    while (bForWIt.hasNext()) {
                                        bForW = bForWIt.next(); 
                                        if (bForW.obj == b.obj) break;
                                    }
                                    int entry = BasicMaths.mod_n(wForB.count - loopSigns, 4);
                                    if ( entry==0 ) { // obtain zero entry
                                        wForBIt.remove(); 
                                        bForWIt.remove();
                                    }
                                    else { // obtain non-zero entry
                                        wForB.count = entry;
                                        bForW.count = entry;
                                    }
                                }
                                else { // w not yet connected to b
                                    Extm4obj bW = new Extm4obj (b.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    Extm4obj wB = new Extm4obj (w.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    w.obj.above.add(bW); 
                                    b.obj.below.add(wB);
                                }      
                            } // end of chaning matrix elements for w-->b
                        } // end of iteration over b
                    } // end of iteration over w
                    for (Iterator<Extm4obj> zIt = y.obj.above.iterator(); zIt.hasNext();) { // Remove y in below lists
                        Extm4obj z = zIt.next();
                        for (Iterator<Extm4obj> yForZIt = z.obj.below.iterator(); yForZIt.hasNext();) {
                            Extm4obj yForZ = yForZIt.next();
                            if (yForZ.obj == y.obj) {
                                yForZIt.remove(); 
                                break;
                            }
                        }      
                    }
                    for (Iterator<m4obj> yyIt = listobj0.iterator(); yyIt.hasNext();) { // Remove y
                        m4obj yy = yyIt.next();
                        if (yy == y.obj) {
                            yyIt.remove(); 
                            break;
                        }
                    }
                } // end of below case
                for (Iterator<Extm4obj> zIt = cand.above.iterator(); zIt.hasNext();) { // Remove cand in below lists
                    Extm4obj z = zIt.next();
                    for (Iterator<Extm4obj> candForZIt = z.obj.below.iterator(); candForZIt.hasNext();) {
                        Extm4obj candForZ = candForZIt.next();
                        if (candForZ.obj == cand) {
                            candForZIt.remove(); 
                            break;
                        }
                    }      
                }
                for (Iterator<Extm4obj> zIt = cand.below.iterator(); zIt.hasNext();) { // Remove cand in above lists
                    Extm4obj z = zIt.next();
                    for (Iterator<Extm4obj> candForZIt = z.obj.above.iterator(); candForZIt.hasNext();) {
                        Extm4obj candForZ = candForZIt.next();
                        if (candForZ.obj == cand) {
                            candForZIt.remove(); 
                            break;
                        }
                    }      
                }
                candIt.remove();
            }
        } 
        
        ArrayList<Cocycle> cocycles = new ArrayList<Cocycle>();
        for(Iterator<Integer> iter = linComb.pos.iterator(); iter.hasNext();) 
        {
            int position = iter.next();
            int helperIndex = linComb.pos.indexOf(position);
            m4obj object = null;
            for (m4obj target : listobj1) {
                if (target.pos == position) {
                    object = target;
                    break;
                }
            }
            if (object.above.size() != 0 && object.below.size() != 0) {
                iter.remove();
                linComb.factor.remove(helperIndex);
            } else if (object.above.size() != object.below.size()) {
                int factor = BasicMaths.mod_n(linComb.factor.get(helperIndex) * 2, 4);
                if (factor == 0) {
                    iter.remove();
                    linComb.factor.remove(helperIndex);
                } else {
                    linComb.factor.set(helperIndex, factor); 
                }
            }
        }
        return linComb;
    }
    
    
    /**
     * Calculates the cocycles that generate the 
     * mod4 homology from chaincomplex information. (still messy)
     */
    ArrayList<Cocycle> getCocycles_4(ArrayList<m4obj> listobj0,
                                     ArrayList<m4obj> listobj1,
                                     ArrayList<m4obj> listobj2) {
        
        Extm4obj y,w;
        m4obj cand;
        Iterator<m4obj> candIt = listobj1.iterator();
        ArrayList<trackObject> trackList = new ArrayList<trackObject>();
        int l1Size = listobj1.size();
        while (candIt.hasNext()) 
        { 
            cand = candIt.next();
            if (cand.above.isEmpty() && cand.below.isEmpty()) continue; 
            
            else 
            { 
                boolean found1or3 = false;
                boolean below = false;
                boolean above = false;
                y = null;
                
                // Find 1' or 3's in boundary matrices
                if (cand.below.size() >= cand.above.size()) 
                {
                    Iterator<Extm4obj> yBelowIt = cand.below.iterator();
                    while (yBelowIt.hasNext()) {        
                        y = yBelowIt.next();
                        if (y.count != 2) {
                            found1or3 = true;
                            below = true;
                            break;
                        }
                    }
                    if (found1or3 == false) {
                        Iterator<Extm4obj> yAboveIt = cand.above.iterator();
                        while (yAboveIt.hasNext()) {
                            y = yAboveIt.next();
                            if (y.count != 2) {
                                found1or3 = true;
                                above = true;
                                break;
                            }
                        }
                    }
                }
                else 
                {
                    Iterator<Extm4obj> yAboveIt = cand.above.iterator();
                    while (yAboveIt.hasNext()) {
                        y = yAboveIt.next();
                        if (y.count != 2) {
                            found1or3 = true;
                            above = true;
                            break;
                        }
                        
                    }
                    if (found1or3 == false) {
                        Iterator<Extm4obj> yBelowIt = cand.below.iterator();
                        while (yBelowIt.hasNext()) {        
                            y = yBelowIt.next();
                            if (y.count != 2) {
                                found1or3 = true;
                                below = true;
                                break;
                            }
                        }
                    }
                }   
                if (found1or3 == false) continue;
                
                // Cancel above
                if (above) 
                { 
                    trackObject tracker = new trackObject(cand.pos, l1Size); // track above-reduction
                    for (int k = 0; k < y.obj.below.size(); k++) {
                        tracker.below[y.obj.below.get(k).obj.pos] = y.obj.below.get(k).count;
                    }
                    trackList.add(tracker);
                    Iterator<Extm4obj> wIt = cand.above.iterator();
                    while (wIt.hasNext()) 
                    { 
                        w = wIt.next();
                        if (w.obj == y.obj) continue;
                        for (Iterator<Extm4obj> bIt = y.obj.below.iterator(); bIt.hasNext();) 
                        { 
                            Extm4obj b = bIt.next();
                            int loopSigns = BasicMaths.mod_n(w.count*y.count*b.count,4);
                            if (loopSigns==0) continue;
                            
                            // Change the matrix entries for b-->w
                            if (b.obj != cand) 
                            {
                                Extm4obj wForB = null;
                                Iterator<Extm4obj> wForBIt = b.obj.above.iterator(); 
                                while (wForBIt.hasNext()) { 
                                    wForB = wForBIt.next();
                                    if (wForB.obj == w.obj) break;
                                }
                                if (wForB.obj == w.obj) { // b already connected to w
                                    Extm4obj bForW = null;
                                    Iterator<Extm4obj> bForWIt = w.obj.below.iterator(); 
                                    while (bForWIt.hasNext()) { 
                                        bForW = bForWIt.next();
                                        if (bForW.obj == b.obj) break;
                                    }
                                    int entry = BasicMaths.mod_n(wForB.count - loopSigns, 4); 
                                    if ( entry==0 ) { // obtain zero entry
                                        wForBIt.remove(); 
                                        bForWIt.remove();
                                    }
                                    else { // obtain non-zero entry
                                        wForB.count = entry;
                                        bForW.count = entry;
                                    }
                                }
                                else { // b not yet connected to w
                                    Extm4obj bW = new Extm4obj (b.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    Extm4obj wB = new Extm4obj (w.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    w.obj.below.add(bW); 
                                    b.obj.above.add(wB);
                                }      
                            } 
                        
                        } 
                    }
                    
                    // Remove y and connections to y
                    for (Iterator<Extm4obj> zIt = y.obj.below.iterator(); zIt.hasNext();) {
                        Extm4obj z = zIt.next();
                        for (Iterator<Extm4obj> yForZIt = z.obj.above.iterator(); yForZIt.hasNext();) {
                            Extm4obj yForZ = yForZIt.next();
                            if (yForZ.obj == y.obj) {
                                yForZIt.remove(); 
                                break;
                            }
                        }      
                    }
                    for (Iterator<m4obj> yyIt = listobj2.iterator(); yyIt.hasNext();) {
                        m4obj yy = yyIt.next();
                        if (yy == y.obj) {
                            yyIt.remove(); 
                            break;
                        }
                    }
                } 
                
                //Cancel below
                else {
                    Iterator<Extm4obj> wIt = cand.below.iterator();
                    while (wIt.hasNext()) 
                    { 
                        w = wIt.next();
                        if (w.obj == y.obj) continue;
                        for (Iterator<Extm4obj> bIt = y.obj.above.iterator(); bIt.hasNext();) 
                        {
                            Extm4obj b = bIt.next();
                            int loopSigns = BasicMaths.mod_n(w.count*y.count*b.count,4);
                            if (loopSigns==0) continue;
                            
                            // Change the matrix entries for w-->b
                            if (b.obj != cand) 
                            {
                                Extm4obj wForB = null;
                                Iterator<Extm4obj> wForBIt = b.obj.below.iterator(); 
                                while (wForBIt.hasNext()) 
                                { 
                                    wForB = wForBIt.next();
                                    if (wForB.obj == w.obj) break;
                                }
                                if (wForB.obj == w.obj) 
                                { 
                                    Extm4obj bForW = null; 
                                    Iterator<Extm4obj> bForWIt = w.obj.above.iterator(); 
                                    while (bForWIt.hasNext()) 
                                    {
                                        bForW = bForWIt.next(); 
                                        if (bForW.obj == b.obj) break;
                                    }
                                    int entry = BasicMaths.mod_n(wForB.count - loopSigns, 4);
                                    if ( entry==0 ) 
                                    {
                                        wForBIt.remove(); 
                                        bForWIt.remove();
                                    }
                                    else 
                                    { 
                                        wForB.count = entry;
                                        bForW.count = entry;
                                    }
                                }
                                else 
                                {
                                    Extm4obj bW = new Extm4obj (b.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    Extm4obj wB = new Extm4obj (w.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    w.obj.above.add(bW); 
                                    b.obj.below.add(wB);
                                }      
                            } 
                        } 
                    } 
                    //Remove y and connections into y
                    for (Iterator<Extm4obj> zIt = y.obj.above.iterator(); zIt.hasNext();) { 
                        Extm4obj z = zIt.next();
                        for (Iterator<Extm4obj> yForZIt = z.obj.below.iterator(); yForZIt.hasNext();) {
                            Extm4obj yForZ = yForZIt.next();
                            if (yForZ.obj == y.obj) {
                                yForZIt.remove(); 
                                break;
                            }
                        }      
                    }
                    for (Iterator<m4obj> yyIt = listobj0.iterator(); yyIt.hasNext();) {
                        m4obj yy = yyIt.next();
                        if (yy == y.obj) {
                            yyIt.remove(); 
                            break;
                        }
                    }
                }
                
                // Remove candidate and connections into candidate
                for (Iterator<Extm4obj> zIt = cand.above.iterator(); zIt.hasNext();) {
                    Extm4obj z = zIt.next();
                    for (Iterator<Extm4obj> candForZIt = z.obj.below.iterator(); candForZIt.hasNext();) {
                        Extm4obj candForZ = candForZIt.next();
                        if (candForZ.obj == cand) {
                            candForZIt.remove(); 
                            break;
                        }
                    }      
                }
                for (Iterator<Extm4obj> zIt = cand.below.iterator(); zIt.hasNext();) {
                    Extm4obj z = zIt.next();
                    for (Iterator<Extm4obj> candForZIt = z.obj.above.iterator(); candForZIt.hasNext();) {
                        Extm4obj candForZ = candForZIt.next();
                        if (candForZ.obj == cand) {
                            candForZIt.remove(); 
                            break;
                        }
                    }      
                }
                candIt.remove();
            } 
        } 
        
        // Remove remaining generators that do not generate the homology
        ArrayList<Cocycle> cocycles = new ArrayList<Cocycle>();
        for(Iterator<m4obj> iter = listobj1.iterator(); iter.hasNext();) 
        {
            m4obj candidate = iter.next();
            if (candidate.above.size() != 0 && candidate.below.size() != 0) iter.remove();
        }
        for(m4obj l1Ele: listobj1) 
        {
            Cocycle a = new Cocycle();
            a.pos.add(l1Ele.pos);
            if (l1Ele.above.size() != l1Ele.below.size()) a.factor.add(2); // valid?
            else a.factor.add(1); // could be a 3?
            cocycles.add(a);
        }
        
        // Recover cocycles in original chaincomplex
        for (Cocycle a : cocycles) 
        {
            for (ListIterator<trackObject> trackIt = trackList.listIterator(trackList.size()); trackIt.hasPrevious();) 
            { 
                trackObject trackInfo = trackIt.previous();
                int candPos = trackInfo.cPos; 
                int coefficient = 0;
                int helps = 0;
                for (Iterator<Integer> factorIt = a.factor.iterator(); factorIt.hasNext();) 
                { 
                    int xPos = a.pos.get(helps);
                    int factor = factorIt.next(); 
                    helps ++;
                    if (trackInfo.below[xPos] == 0) continue;  
                    coefficient += -1*factor*trackInfo.below[xPos]*trackInfo.below[candPos];
                }
                coefficient = BasicMaths.mod_n(coefficient, 4);
                if (coefficient == 0) continue;
                else 
                { 
                    a.pos.add(candPos);
                    a.factor.add(coefficient);
                }
            }
        }
        return cocycles;   
    }
    
    /**
     * Simplifies chaincomplex preserving homology. (still messy)
     */
    void simplifyChainCx_4(ArrayList<m4obj> listobj0,
                        ArrayList<m4obj> listobj1,
                        ArrayList<m4obj> listobj2) {
        
        Extm4obj y,w;
        m4obj cand;
        Iterator<m4obj> candIt = listobj1.iterator();
        while (candIt.hasNext()) { // Iterate through candidates
            cand = candIt.next();
            if (cand.above.isEmpty() && cand.below.isEmpty()) {
                continue; 
            }
            else { // Reduction step
                boolean found1or3 = false;
                boolean below = false;
                boolean above = false;
                y = null;
                if (cand.below.size() >= cand.above.size()) { // Find 1's or 3's, prefer larger side
                    Iterator<Extm4obj> yBelowIt = cand.below.iterator();
                    while (yBelowIt.hasNext()) {        
                        y = yBelowIt.next();
                        if (y.count != 2) {
                            found1or3 = true;
                            below = true;
                            break;
                        }
                    }
                    if (found1or3 == false) {
                        Iterator<Extm4obj> yAboveIt = cand.above.iterator();
                        while (yAboveIt.hasNext()) {
                            y = yAboveIt.next();
                            if (y.count != 2) {
                                found1or3 = true;
                                above = true;
                                break;
                            }
                        }
                    }
                }
                else {
                    Iterator<Extm4obj> yAboveIt = cand.above.iterator();
                    while (yAboveIt.hasNext()) {
                        y = yAboveIt.next();
                        if (y.count != 2) {
                            found1or3 = true;
                            above = true;
                            break;
                        }
                    }
                    if (found1or3 == false) {
                        Iterator<Extm4obj> yBelowIt = cand.below.iterator();
                        while (yBelowIt.hasNext()) {        
                            y = yBelowIt.next();
                            if (y.count != 2) {
                                found1or3 = true;
                                below = true;
                                break;
                            }
                        }
                    }
                }   
                if (found1or3 == false) continue;
                if (above) { // 2nd cadidate, y,lies above 
                    if (cand.above.size() > 2) { 
                    }
                    Iterator<Extm4obj> wIt = cand.above.iterator();
                    while (wIt.hasNext()) { // Iterate through cand's connections above (other than y), w 
                        w = wIt.next();
                        if (w.obj == y.obj) continue;
                        for (Iterator<Extm4obj> bIt = y.obj.below.iterator(); bIt.hasNext();) { // Iterate through y's connections below, b
                            Extm4obj b = bIt.next();
                            int loopSigns = BasicMaths.mod_n(w.count*y.count*b.count,4);
                            if (loopSigns==0) continue;
                            if (b.obj != cand) { // Change the matrix entries for b-->w
                                Extm4obj wForB = null;
                                Iterator<Extm4obj> wForBIt = b.obj.above.iterator(); 
                                while (wForBIt.hasNext()) { 
                                    wForB = wForBIt.next();
                                    if (wForB.obj == w.obj) break;
                                }
                                if (wForB.obj == w.obj) { // b already connected to w
                                    Extm4obj bForW = null;
                                    Iterator<Extm4obj> bForWIt = w.obj.below.iterator(); 
                                    while (bForWIt.hasNext()) { 
                                        bForW = bForWIt.next();
                                        if (bForW.obj == b.obj) break;
                                    }
                                    int entry = BasicMaths.mod_n(wForB.count - loopSigns, 4); 
                                    if ( entry==0 ) { // obtain zero entry
                                        wForBIt.remove(); 
                                        bForWIt.remove();
                                    }
                                    else { // obtain non-zero entry
                                        wForB.count = entry;
                                        bForW.count = entry;
                                    }
                                }
                                else { // b not yet connected to w
                                    Extm4obj bW = new Extm4obj (b.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    Extm4obj wB = new Extm4obj (w.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    w.obj.below.add(bW); 
                                    b.obj.above.add(wB);
                                }      
                            } // end of changing matrix elements for b-->w   
                        } // end of iteration over b
                    } // end of iteration over w
                    for (Iterator<Extm4obj> zIt = y.obj.below.iterator(); zIt.hasNext();) { // Remove y in above lists
                        Extm4obj z = zIt.next();
                        for (Iterator<Extm4obj> yForZIt = z.obj.above.iterator(); yForZIt.hasNext();) {
                            Extm4obj yForZ = yForZIt.next();
                            if (yForZ.obj == y.obj) {
                                yForZIt.remove(); 
                                break;
                            }
                        }      
                    }
                    for (Iterator<m4obj> yyIt = listobj2.iterator(); yyIt.hasNext();) { // Remove y
                        m4obj yy = yyIt.next();
                        if (yy == y.obj) {
                            yyIt.remove(); 
                            break;
                        }
                    }
                } 
                
                else { // 2nd canditate lies below, same code as for "above case"
                    Iterator<Extm4obj> wIt = cand.below.iterator();
                    while (wIt.hasNext()) { // Iterate through cand's connections below (other than y), w 
                        w = wIt.next();
                        if (w.obj == y.obj) continue;
                        for (Iterator<Extm4obj> bIt = y.obj.above.iterator(); bIt.hasNext();) { // Iterate through y's connections above, b
                            Extm4obj b = bIt.next();
                            int loopSigns = BasicMaths.mod_n(w.count*y.count*b.count,4);
                            if (loopSigns==0) continue;
                            if (b.obj != cand) { // Change the matrix entries for w-->b
                                Extm4obj wForB = null;
                                Iterator<Extm4obj> wForBIt = b.obj.below.iterator(); // check whether and where w is in b.obj.below
                                while (wForBIt.hasNext()) { 
                                    wForB = wForBIt.next();
                                    if (wForB.obj == w.obj) break;
                                }
                                if (wForB.obj == w.obj) { // w already connected to b
                                    Extm4obj bForW = null; 
                                    Iterator<Extm4obj> bForWIt = w.obj.above.iterator(); 
                                    while (bForWIt.hasNext()) {
                                        bForW = bForWIt.next(); 
                                        if (bForW.obj == b.obj) break;
                                    }
                                    int entry = BasicMaths.mod_n(wForB.count - loopSigns, 4);
                                    if ( entry==0 ) { // obtain zero entry
                                        wForBIt.remove(); 
                                        bForWIt.remove();
                                    }
                                    else { // obtain non-zero entry
                                        wForB.count = entry;
                                        bForW.count = entry;
                                    }
                                }
                                else { // w not yet connected to b
                                    Extm4obj bW = new Extm4obj (b.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    Extm4obj wB = new Extm4obj (w.obj,BasicMaths.mod_n(-1*loopSigns, 4));
                                    w.obj.above.add(bW); 
                                    b.obj.below.add(wB);
                                }      
                            } // end of chaning matrix elements for w-->b
                        } // end of iteration over b
                    } // end of iteration over w
                    for (Iterator<Extm4obj> zIt = y.obj.above.iterator(); zIt.hasNext();) { // Remove y in below lists
                        Extm4obj z = zIt.next();
                        for (Iterator<Extm4obj> yForZIt = z.obj.below.iterator(); yForZIt.hasNext();) {
                            Extm4obj yForZ = yForZIt.next();
                            if (yForZ.obj == y.obj) {
                                yForZIt.remove(); 
                                break;
                            }
                        }      
                    }
                    for (Iterator<m4obj> yyIt = listobj0.iterator(); yyIt.hasNext();) { // Remove y
                        m4obj yy = yyIt.next();
                        if (yy == y.obj) {
                            yyIt.remove(); 
                            break;
                        }
                    }
                } // end of below case
                for (Iterator<Extm4obj> zIt = cand.above.iterator(); zIt.hasNext();) { // Remove cand in below lists
                    Extm4obj z = zIt.next();
                    for (Iterator<Extm4obj> candForZIt = z.obj.below.iterator(); candForZIt.hasNext();) {
                        Extm4obj candForZ = candForZIt.next();
                        if (candForZ.obj == cand) {
                            candForZIt.remove(); 
                            break;
                        }
                    }      
                }
                for (Iterator<Extm4obj> zIt = cand.below.iterator(); zIt.hasNext();) { // Remove cand in above lists
                    Extm4obj z = zIt.next();
                    for (Iterator<Extm4obj> candForZIt = z.obj.above.iterator(); candForZIt.hasNext();) {
                        Extm4obj candForZ = candForZIt.next();
                        if (candForZ.obj == cand) {
                            candForZIt.remove(); 
                            break;
                        }
                    }      
                }
                candIt.remove(); // Remove cand
            } // end of reduction step
        } // end of candidate iteration
        
    }
     
    /**
     * 2 methods to extract chaincomplex information
     * from flow category information (mod4, mod0)
     */
    void createM4mods(ArrayList<flow_object> obs1, ArrayList<flow_object> obs0,
                      ArrayList<m4obj> list1, ArrayList<m4obj> list0) {
        int v;
        for (int u = 0; u < obs1.size(); u++) {
            ArrayList<Integer> helpers = new ArrayList<Integer>();
            ArrayList<Integer> helpersCount = new ArrayList<Integer>();
            for (mod0 mod : obs1.get(u).moduli0t) {
                v = obs0.indexOf(mod.bot);
                if (helpers.contains((Integer)v)) {
                    int Count = 0; 
                    if (mod.sign) Count = 1;
                    else Count = -1;
                    int w = helpersCount.get(helpers.indexOf(v));
                    w = BasicMaths.mod_n(w + Count, 4);
                    if (w == 0) {
                        helpersCount.remove(helpers.indexOf(v));
                        helpers.remove((Integer) v);
                    }
                    else helpersCount.set(helpers.indexOf(v), w);
                }
                else { 
                    helpers.add(v);
                    int Count = 0;
                    if (mod.sign) Count = 1;
                    else Count = 3;
                    helpersCount.add(Count);
                }
            }
            for (int x : helpers) {
                Extm4obj holder1 = new Extm4obj(list0.get(x), helpersCount.get(helpers.indexOf(x)));
                list1.get(u).below.add((Extm4obj) holder1);
                Extm4obj holder2 = new Extm4obj(list1.get(u), helpersCount.get(helpers.indexOf(x)));
                list0.get(x).above.add(holder2);
            }
        }
    }
    void createM0Mods(ArrayList<flow_object> obs1, ArrayList<flow_object> obs0,
                      ArrayList<m4obj> list1, ArrayList<m4obj> list0) {
        int v;
        for (int u = 0; u < obs1.size(); u++) {
            ArrayList<Integer> helpers = new ArrayList<Integer>();
            ArrayList<Integer> helpersCount = new ArrayList<Integer>();
            for (mod0 mod : obs1.get(u).moduli0t) {
                v = obs0.indexOf(mod.bot);
                if (helpers.contains((Integer)v)) {
                    int Count = -1; 
                    if (mod.sign) Count = 1;
                    int w = helpersCount.get(helpers.indexOf(v));
                    w = w+Count;
                    if (w == 0) {
                        helpersCount.remove(helpers.indexOf(v));
                        helpers.remove((Integer) v);
                    }
                    else helpersCount.set(helpers.indexOf(v), w);
                }
                else { 
                    helpers.add(v);
                    int Count = 0;
                    if (mod.sign) Count = 1;
                    else Count = -1;
                    helpersCount.add(Count);
                }
            }
            for (int x : helpers) {
                Extm4obj holder1 = new Extm4obj(list0.get(x), helpersCount.get(helpers.indexOf(x)));
                list1.get(u).below.add((Extm4obj) holder1);
                Extm4obj holder2 = new Extm4obj(list1.get(u), helpersCount.get(helpers.indexOf(x)));
                list0.get(x).above.add(holder2);
            }
        }
    }
    
    
    /**
     * 3 methods to restore or initiate chaincomplex information
     * from flow category information (mod2 ,mod4, mod0)
     */
    void resetChainCx_2(ArrayList<flow_object> objects0, ArrayList<m2obj> listobj0,
                        ArrayList<flow_object> objects1, ArrayList<m2obj> listobj1,
                        ArrayList<flow_object> objects2, ArrayList<m2obj> listobj2) {
        
        listobj0.clear();
        listobj1.clear();
        listobj2.clear();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        createM2mods(objects1, objects0, listobj1, listobj0);
        createM2mods(objects2, objects1, listobj2, listobj1);
    }   
    void resetChainCx_4(ArrayList<flow_object> objects0, ArrayList<m4obj> listobj0,
                        ArrayList<flow_object> objects1, ArrayList<m4obj> listobj1,
                        ArrayList<flow_object> objects2, ArrayList<m4obj> listobj2) {
        
        listobj0.clear();
        listobj1.clear();
        listobj2.clear();
        for (int u = 0; u < objects0.size(); u++) {
            m4obj help = new m4obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m4obj help = new m4obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m4obj help = new m4obj(u);
            listobj2.add(help);
        }
        createM4mods(objects1, objects0, listobj1, listobj0);
        createM4mods(objects2, objects1, listobj2, listobj1);
        
    }
    void resetChainCx_0(ArrayList<flow_object> objects0, ArrayList<m4obj> listobj0,
                        ArrayList<flow_object> objects1, ArrayList<m4obj> listobj1,
                        ArrayList<flow_object> objects2, ArrayList<m4obj> listobj2) {
        
        listobj0.clear();
        listobj1.clear();
        listobj2.clear();
        for (int u = 0; u < objects0.size(); u++) {
            m4obj help = new m4obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m4obj help = new m4obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m4obj help = new m4obj(u);
            listobj2.add(help);
        }
        createM0Mods(objects1, objects0, listobj1, listobj0);
        createM0Mods(objects2, objects1, listobj2, listobj1);
        
    }
    
    
    /**
     * Determines mod4 (or higher) boundary matrix 
     * from chaincomplex information.
     */
    public static int[][] detBoundMat(ArrayList<m4obj> list0, ArrayList<m4obj> list1) {
        int[][] matrix;
        if (list1.size() == 0 && list0.size() != 0) { matrix = new int[1][list0.size()]; }
        else if (list1.size() != 0 && list0.size() == 0) { matrix = new int[list1.size()][1]; }
        else if (list1.size() == 0 && list0.size() == 0) { matrix = new int[1][1]; }
        else {
            matrix = new int[list1.size()][list0.size()];
            for (Iterator<m4obj> l1It = list1.iterator(); l1It.hasNext();) {
                m4obj l1Ele = l1It.next();
                int helperIx = list1.indexOf(l1Ele);
                for (Iterator<Extm4obj> belowIt = l1Ele.below.iterator(); belowIt.hasNext();) {
                    Extm4obj below = belowIt.next();
                    matrix[helperIx][list0.indexOf(below.obj)] = below.count;
                }
            }
        }
        return matrix;
    }
    
    
    /**
     * Determines mod2 (or higher) boundary matrix 
     * from chaincomplex information.
     */
    public static int[][] detBoundMat_2(ArrayList<m2obj> list0, ArrayList<m2obj> list1) {
        int[][] matrix;
        if (list1.size() == 0 && list0.size() != 0) { matrix = new int[1][list0.size()]; }
        else if (list1.size() != 0 && list0.size() == 0) { matrix = new int[list1.size()][1]; }
        else if (list1.size() == 0 && list0.size() == 0) { matrix = new int[1][1]; }
        else {
            matrix = new int[list1.size()][list0.size()];
            for (Iterator<m2obj> l1It = list1.iterator(); l1It.hasNext();) {
                m2obj l1Ele = l1It.next();
                int helperIx = list1.indexOf(l1Ele);
                for (Iterator<m2obj> belowIt = l1Ele.below.iterator(); belowIt.hasNext();) {
                    m2obj below = belowIt.next();
                    matrix[helperIx][below.pos] = 1;
                }
            }
        }
        return matrix; 
    }
    
    
    /**
     * Converts mod2 cocycles into form
     * compatible with mod4 cocycles.
     */
    ArrayList<Cocycle> convertCocycles(ArrayList<ArrayList<Integer>> input) {
        
        ArrayList<Cocycle> output = new  ArrayList<Cocycle>();
        for (ArrayList<Integer> cocy : input) {
            Cocycle gener = new Cocycle();
            for(int i : cocy) {
                gener.pos.add(i);
                gener.factor.add(1);
            }
            output.add(gener);
        }
        return output;  
    }
    
    /**
     * Converts mod4-form cocycles into mod2 form.
     */
    ArrayList<ArrayList<Integer>> convertBackCocycles(ArrayList<Cocycle> input) {
     
        ArrayList<ArrayList<Integer>> output = new ArrayList<ArrayList<Integer>>();
        for (Cocycle cocy : input) {
            ArrayList<Integer> gener = new ArrayList<Integer>();
            for(int i : cocy.pos) {
                gener.add(i);
            }
            output.add(gener);
        }
        return output;  
    }
    
    
    /**
     * Determines whether a linear combination of
     * generators represents a cocycle.
     */
    boolean checkCocycle(Cocycle cocycle, int[][] Matrix, int mod) {
        int[][] col = new int[Matrix[0].length][1];
        for (int i : cocycle.pos) {
            col[i][0] = cocycle.factor.get(cocycle.pos.indexOf(i));
        }
        int[][] result = IntegerMatrices.mult_n(Matrix, col, mod);
        int count = 0;
        boolean checker;
        for (int k = 0; k< result.length; k++) {
            if (result[k][0] !=0) count += result[k][0];
        }
        checker = (count > 0) ? false :  true;
        return checker;
        
    }
    
  
    private class trackObject {
        int cPos;
        int[] below;
        
        trackObject(int candPos, int size) {
            cPos = candPos;
            below = new int[size];
        }
    }
    
    private class Cocycle {
        ArrayList<Integer> pos;
        ArrayList<Integer> factor;
        
        Cocycle() {
            pos = new ArrayList<Integer>();
            factor = new ArrayList<Integer>();
        }
        
        void print(boolean factr) {
            //System.out.print("    > ");
            for (int i = 0; i< pos.size(); i++) {
                //if (i > 0 && i % 20 == 0) {
                //    System.out.println();
                //    System.out.print("      ");
                //}
                if (factr) System.out.print(pos.get(i) + "(" + factor.get(i) + ")" + " ");
                else System.out.print(pos.get(i) + " ");
            }
            System.out.println();
        }
    }
        
    private class m4obj {
        int pos;
        ArrayList<Extm4obj> below;
        ArrayList<Extm4obj> above;
        
        m4obj(int p) {
            pos = p;
            below = new ArrayList<Extm4obj>();
            above = new ArrayList<Extm4obj>();
        }
    }
    
    private class Extm4obj {
        m4obj obj;
        int count;
        
        Extm4obj(m4obj m4oj, int c) {
            obj = m4oj;
            count = c;
        }
    }
    
    
    
    
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////    
    
    
    
    
    
    final link doubleGauss(String arg) {
     link Link = null;
  int fool = 0;
     int [] crossings = new int[3];
     int [][] paths = new int[3][4];
     int k = arg.length();
  ArrayList<Integer> commas = new ArrayList<Integer>();
  ArrayList<Integer> dots = new ArrayList<Integer>();
  int j = 1;
  while ( j < k ) {
   if (arg.charAt(j) == ',') commas.add(j);
   if (arg.charAt(j) == '.') dots.add(j);
   j++;
  }
  int test = dots.size()+ commas.size();
  if (test %2 != 0) fool = 5;
  else {
   crossings = new int [(test)/2];
   paths = new int [(test)/2][4];
   int c = 0;
   int d = 0;
   boolean commainc = true;
   try {
    int pos = 0;
    int nextpos;
    int value;
    int absvalue;
    int path = 1;
    int startpath = 1;
    while ((c+d) < test) {
     if (c < commas.size()) {
      if (commas.get(c) < dots.get(d)) {
       nextpos = commas.get(c);
       c++;
       commainc = true;
      }
      else {
       nextpos = dots.get(d);
       d++;
       commainc = false;
      }
     }
     else {
      nextpos = dots.get(d);
      d++;
      commainc = false;
     }
     value = Integer.parseInt(arg.substring(pos,nextpos));
     if (commainc) pos = commas.get(c-1)+1;
     else pos = dots.get(d-1)+1;
     absvalue = Math.abs(value)-1;
     if (crossings[absvalue] != 0) {
      if (paths[absvalue][2] != 0) fool = 4;
      else {
       int a = 2;
       int b = 3;
       if ((crossings[absvalue] * value) < 0) {
        crossings[absvalue] = -crossings[absvalue];
        a = 1;
        b = 0;
        paths[absvalue][2] = paths[absvalue][1];
        paths[absvalue][3] = paths[absvalue][0];
       }
       paths[absvalue][a] = path;
       path++;
       if (commainc) paths[absvalue][b] = path;
       else {
        paths[absvalue][b] = startpath;
        startpath = path;
       }
      }
     }
     else {
      if (value < 0) crossings[absvalue] = -2;
      else crossings[absvalue] = 2;
      paths[absvalue][0] = path;
      path++;
      if (commainc) paths[absvalue][1] = path;
      else {
       paths[absvalue][1] = startpath;
       startpath = path;
      }
     }
    }
   }
   catch (NumberFormatException e) {
    fool = 4;
   }
  }
  if (fool == 0) Link = new link(crossings,paths);
  return Link;
    }
    
    final link diagramCreator(String arg1, String arg2) {
     link Link = null;
     int fool = 0;
     int [] crossings = new int[3];
     int [][] paths = new int[3][4];
     arg1 = arg1+",";
        int k = arg1.length();
        ArrayList<Integer> commas = new ArrayList<Integer>();
        int j = 1;
        while (j < k) {
            if (arg1.charAt(j) == ',') commas.add(j);
            j++;
        }
        crossings = new int [commas.size()];
        try {
            int pos = 0;
            for (int i = 0 ; i < commas.size(); i++) {
                crossings[i] = Integer.parseInt(arg1.substring(pos,commas.get(i)));
                pos = commas.get(i)+1;
            }
        }
        catch (NumberFormatException e) {
            fool = 4;
        }
        arg2 = arg2+",";
        k = arg2.length();
        commas.clear();
        j = 1;
        while (j < k) {
            if (arg2.charAt(j) == ',') commas.add(j);
            j++;
        }
        paths = new int [crossings.length][4];
        if (commas.size() == 4 * crossings.length) {
            try {
                int pos = 0;
                for (int i = 0; i < commas.size(); i++) {
                    paths[i/4][i%4] = Integer.parseInt(arg2.substring(pos,commas.get(i)));
                    pos = commas.get(i)+1;
                }
            }
            catch (Exception e) {
                fool = 5;
            }
        }
        else fool = 5;
        if (fool == 0) Link = new link(crossings,paths);
     return Link;
    }
    
    final void SquareOne(link Link, int hdeg, int qdeg, int n) {
     
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        objects0 = getObjects(Link, hdeg-1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg+1, qdeg, n);
        objects3 = getObjects(Link, hdeg+2, qdeg, n);
        get0Mod(objects1,objects0,Link.paths,n);
        get0Mod(objects2,objects1,Link.paths,n);
        get0Mod(objects3,objects2,Link.paths,n);
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj3 = new ArrayList<m2obj>();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects1,objects0,listobj1,listobj0);
        createM2mods(objects2,objects1,listobj2,listobj1);
        ArrayList<ArrayList<Integer>> cocycles;
        cocycles = getCocycles(listobj0,listobj1,listobj2);
        for (int u = 0; u < cocycles.size(); u++) {
            System.out.print("Cocycle : ");
            for (int v : cocycles.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        ArrayList<ArrayList<Integer>> bocks = Bockstein(cocycles,objects1,objects2);
        for (int u = 0; u < bocks.size(); u++) {
         System.out.print("Bockstein : ");
            for (int v : bocks.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        listobj2.clear();
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        listobj1.clear();
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        createM2mods(objects2,objects1,listobj2,listobj1);
        createM2mods(objects3,objects2,listobj3,listobj2);
        ShowHomCls(bocks,listobj1,listobj2,listobj3);
    }
    
    final void SquareTwo(link Link, int hdeg,int qdeg, int n) {
     
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<flow_object> objects4;
        objects0 = getObjects(Link, hdeg-1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg+1, qdeg, n);
        objects3 = getObjects(Link, hdeg+2, qdeg, n);
        get0Mod(objects1,objects0,Link.paths,n);
        get0Mod(objects2,objects1,Link.paths,n);
        get0Mod(objects3,objects2,Link.paths,n);
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj3 = new ArrayList<m2obj>();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects1,objects0,listobj1,listobj0);
        createM2mods(objects2,objects1,listobj2,listobj1);
        ArrayList<ArrayList<Integer>> cocycles;
        cocycles = getCocycles(listobj0,listobj1,listobj2);
        for (int u = 0; u < cocycles.size(); u++) {
            System.out.print("Cocycle : ");
            for (int v : cocycles.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        ArrayList<ArrayList<Integer>> steensq = 
                Steenrod(cocycles,objects1,objects2,objects3,Link);
        for (int u = 0; u < steensq.size(); u++) {
            System.out.print("Steenrod : ");
            for (int v : steensq.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        objects4 = getObjects(Link,hdeg+3, qdeg, n);
        get0Mod(objects4,objects3,Link.paths, n);
        ArrayList<m2obj> listobj4 = new ArrayList<m2obj>();
        for (int u = 0; u < objects4.size(); u++) {
            m2obj help = new m2obj(u);
            listobj4.add(help);
        } 
        listobj3.clear();
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        listobj2.clear();
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        createM2mods(objects3,objects2,listobj3,listobj2);
        createM2mods(objects4,objects3,listobj4,listobj3);
        ShowHomCls(steensq,listobj2,listobj3,listobj4);
    }
    
    final void SquareThree(link Link, int hdeg,int qdeg, int n) {
     
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<flow_object> objects4;
        ArrayList<flow_object> objects5;
        objects0 = getObjects(Link, hdeg-1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg+1, qdeg, n);
        objects3 = getObjects(Link, hdeg+2, qdeg, n);
        get0Mod(objects1,objects0,Link.paths,n);
        get0Mod(objects2,objects1,Link.paths,n);
        get0Mod(objects3,objects2,Link.paths,n);
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj3 = new ArrayList<m2obj>();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects1,objects0,listobj1,listobj0);
        createM2mods(objects2,objects1,listobj2,listobj1);
        ArrayList<ArrayList<Integer>> cocycles;
        cocycles = getCocycles(listobj0,listobj1,listobj2);
        for (int u = 0; u < cocycles.size(); u++) {
            System.out.print("Cocycle : ");
            for (int v : cocycles.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        ArrayList<ArrayList<Integer>> steensq = 
                Steenrod(cocycles,objects1,objects2,objects3,Link);
        for (int u = 0; u < steensq.size(); u++) {
            System.out.print("Steenrod : ");
            for (int v : steensq.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        objects4 = getObjects(Link,hdeg+3, qdeg, n);
        get0Mod(objects4,objects3,Link.paths, n);
        ArrayList<ArrayList<Integer>> bocks = Bockstein(steensq,objects3,objects4);
        for (int u = 0; u < bocks.size(); u++) {
         System.out.print("Bockstein : ");
            for (int v : bocks.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        objects5 = getObjects(Link,hdeg+4, qdeg, n);
        get0Mod(objects5,objects4,Link.paths, n);
        ArrayList<m2obj> listobj4 = new ArrayList<m2obj>();
        for (int u = 0; u < objects4.size(); u++) {
            m2obj help = new m2obj(u);
            listobj4.add(help);
        }
        ArrayList<m2obj> listobj5 = new ArrayList<m2obj>();
        for (int u = 0; u < objects5.size(); u++) {
            m2obj help = new m2obj(u);
            listobj5.add(help);
        }
        listobj3.clear();
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects4,objects3,listobj4,listobj3);
        createM2mods(objects5,objects4,listobj5,listobj4);
        ShowHomCls(bocks,listobj3,listobj4,listobj5);
    }
    
    final void SquareTwoOne(link Link, int hdeg, int qdeg, int n) {
     
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<flow_object> objects4;
        ArrayList<flow_object> objects5;
        objects0 = getObjects(Link, hdeg-1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg+1, qdeg, n);
        objects3 = getObjects(Link, hdeg+2, qdeg, n);
        get0Mod(objects1,objects0,Link.paths,n);
        get0Mod(objects2,objects1,Link.paths,n);
        get0Mod(objects3,objects2,Link.paths,n);
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj3 = new ArrayList<m2obj>();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects1,objects0,listobj1,listobj0);
        createM2mods(objects2,objects1,listobj2,listobj1);
        ArrayList<ArrayList<Integer>> cocycles;
        cocycles = getCocycles(listobj0,listobj1,listobj2);
        for (int u = 0; u < cocycles.size(); u++) {
            System.out.print("Cocycle : ");
            for (int v : cocycles.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        ArrayList<ArrayList<Integer>> bocks = Bockstein(cocycles,objects1,objects2);
        for (int u = 0; u < bocks.size(); u++) {
         System.out.print("Bockstein : ");
            for (int v : bocks.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        objects4 = getObjects(Link,hdeg+3, qdeg, n);
        get0Mod(objects4,objects3,Link.paths, n);
        ArrayList<ArrayList<Integer>> steensq = 
                Steenrod(bocks,objects2,objects3,objects4,Link);
        for (int u = 0; u < steensq.size(); u++) {
            System.out.print("Steenrod : ");
            for (int v : steensq.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        objects5 = getObjects(Link,hdeg+4, qdeg, n);
        get0Mod(objects5,objects4,Link.paths, n);
        ArrayList<m2obj> listobj4 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj5 = new ArrayList<m2obj>();
        for (int u = 0; u < objects4.size(); u++) {
            m2obj help = new m2obj(u);
            listobj4.add(help);
        }
        for (int u = 0; u < objects5.size(); u++) {
            m2obj help = new m2obj(u);
            listobj5.add(help);
        }
        createM2mods(objects4,objects3,listobj4,listobj3);
        createM2mods(objects5,objects4,listobj5,listobj4);
        ShowHomCls(steensq,listobj3,listobj4,listobj5);
    }
    
    final void SquareTwoTwo(link Link, int hdeg, int qdeg, int n) {
     
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<flow_object> objects4;
        ArrayList<flow_object> objects5;
        ArrayList<flow_object> objects6;
        objects0 = getObjects(Link, hdeg-1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg+1, qdeg, n);
        objects3 = getObjects(Link, hdeg+2, qdeg, n);
        get0Mod(objects1,objects0,Link.paths,n);
        get0Mod(objects2,objects1,Link.paths,n);
        get0Mod(objects3,objects2,Link.paths,n);
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj3 = new ArrayList<m2obj>();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects1,objects0,listobj1,listobj0);
        createM2mods(objects2,objects1,listobj2,listobj1);
        ArrayList<ArrayList<Integer>> cocycles;
        cocycles = getCocycles(listobj0,listobj1,listobj2);
        for (int u = 0; u < cocycles.size(); u++) {
            System.out.print("Cocycle : ");
            for (int v : cocycles.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        ArrayList<ArrayList<Integer>> steensq = 
                Steenrod(cocycles,objects1,objects2,objects3,Link);
        for (int u = 0; u < steensq.size(); u++) {
            System.out.print("Steenrod : ");
            for (int v : steensq.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        objects4 = getObjects(Link,hdeg+3, qdeg, n);
        get0Mod(objects4,objects3,Link.paths, n);
        objects5 = getObjects(Link,hdeg+4, qdeg, n);
        get0Mod(objects5,objects4,Link.paths, n);
        ArrayList<ArrayList<Integer>> steensqu = 
          Steenrod(steensq,objects3,objects4,objects5,Link);
        for (int u = 0; u < steensqu.size(); u++) {
         System.out.print("Steenrod : ");
            for (int v : steensqu.get(u)) System.out.print(v+" ");
            System.out.println();
        }
        objects6 = getObjects(Link,hdeg+5, qdeg, n);
        get0Mod(objects6,objects5,Link.paths, n);
        ArrayList<m2obj> listobj4 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj5 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj6 = new ArrayList<m2obj>();
        for (int u = 0; u < objects4.size(); u++) {
            m2obj help = new m2obj(u);
            listobj4.add(help);
        }
        for (int u = 0; u < objects5.size(); u++) {
            m2obj help = new m2obj(u);
            listobj5.add(help);
        }
        
        for (int u = 0; u < objects6.size(); u++) {
            m2obj help = new m2obj(u);
            listobj6.add(help);
        }
        createM2mods(objects5,objects4,listobj5,listobj4);
        createM2mods(objects6,objects5,listobj6,listobj5);
        ShowHomCls(steensqu,listobj4,listobj5,listobj6);
    }
    
    final void SteenSq(link Link, int hdeg,int qdeg,int n) {
        
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        ArrayList<flow_object> objects3;
        ArrayList<flow_object> objects4;
        objects0 = getObjects(Link, hdeg-1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg+1, qdeg, n);
        
        get0Mod(objects1,objects0,Link.paths,n);
        get0Mod(objects2,objects1,Link.paths,n);
        
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        createM2mods(objects1,objects0,listobj1,listobj0);
        createM2mods(objects2,objects1,listobj2,listobj1);
        ArrayList<ArrayList<Integer>> cocycles;
        cocycles = getCocycles(listobj0,listobj1,listobj2);
        if (cocycles.isEmpty()) {
         System.out.println("St("+hdeg+","+qdeg+","+n+") = ( 0, 0, 0, 0)");
         return;
        }
        ArrayList<ArrayList<Integer>> bocks = Bockstein(cocycles,objects1,objects2);
        listobj2.clear();
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        listobj1.clear();
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        objects3 = getObjects(Link, hdeg+2, qdeg, n);
        get0Mod(objects3,objects2,Link.paths,n);
        ArrayList<m2obj> listobj3 = new ArrayList<m2obj>();
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects2,objects1,listobj2,listobj1);
        createM2mods(objects3,objects2,listobj3,listobj2);
        ArrayList<ArrayList<Integer>> kercycles;
        kercycles = obtainKerBock(cocycles,bocks,listobj1,listobj2,listobj3);
        boolean ker;
        if (kercycles.isEmpty()) ker = false;
        else ker = true;
        int a = cocycles.size();
        if (ker & (kercycles.size()!=cocycles.size()))
         for (ArrayList<Integer> kern : kercycles) cocycles.add(kern);
        int b = cocycles.size();
        ArrayList<ArrayList<Integer>> steensq = 
                Steenrod(cocycles,objects1,objects2,objects3,Link);
        ArrayList<ArrayList<Integer>> bocycles;
        listobj1.clear();
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        listobj2.clear();
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        listobj3.clear();
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        createM2mods(objects2,objects1,listobj2,listobj1);
        createM2mods(objects3,objects2,listobj3,listobj2);
        bocycles = getCocycles(listobj1,listobj2,listobj3);
        ArrayList<ArrayList<Integer>> bockt = Bockstein(bocycles,objects2,objects3);
        for (ArrayList<Integer> bock : bockt) steensq.add(bock);
        objects4 = getObjects(Link,hdeg+3, qdeg, n);
        get0Mod(objects4,objects3,Link.paths, n);
        ArrayList<m2obj> listobj4 = new ArrayList<m2obj>();
        for (int u = 0; u < objects4.size(); u++) {
            m2obj help = new m2obj(u);
            listobj4.add(help);
        } 
        listobj3.clear();
        for (int u = 0; u < objects3.size(); u++) {
            m2obj help = new m2obj(u);
            listobj3.add(help);
        }
        listobj2.clear();
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        createM2mods(objects3,objects2,listobj3,listobj2);
        createM2mods(objects4,objects3,listobj4,listobj3);
        System.out.print("St("+hdeg+","+qdeg+","+n+") = ");
        obtainRanks(steensq,listobj2,listobj3,listobj4,ker,a,b);
    }
    
    void obtainRanks(ArrayList<ArrayList<Integer>> cocyc, ArrayList<m2obj> listobj0,
            ArrayList<m2obj> listobj1, ArrayList<m2obj> listobj2, boolean ker,
            int a, int b) {
     int r1,r2,r3,r4, h1,h2;
     int x1,x2,x3,x4;
     DetHomCls(cocyc, listobj0,listobj1,listobj2);
     ArrayList<ArrayList<Boolean>> matrix = new ArrayList<ArrayList<Boolean>>();
     for (int u = 0; u < a; u++) {
      ArrayList<Boolean> line = new ArrayList<Boolean>();
      for (int v = 0; v < listobj1.size(); v++) {
       if (cocyc.get(u).contains(listobj1.get(v).pos)) line.add(false);
       else line.add(true);
      }
      matrix.add(line);
     }
     r1 = rank(matrix);
     for (int w = b; w < cocyc.size(); w++) {
      ArrayList<Boolean> line = new ArrayList<Boolean>();
      for (int v = 0; v < listobj1.size(); v++) {
       if (cocyc.get(w).contains(listobj1.get(v).pos)) line.add(false);
       else line.add(true);
      }
      matrix.add(line);
     }
     h1 = rank(matrix);
     for (int u = a-1; u >= 0; u--) matrix.remove(u);
     h2 = rank(matrix);
     r3 = h2+r1-h1;
     if (!ker) {
      r2 = 0;
      r4 = 0;
     }
     else {
      if (ker & (a==b)) {
       r2 = r1;
       r4 = r3;
      }
      else {
       matrix.clear();
       for (int u = a; u < b; u++) {
           ArrayList<Boolean> line = new ArrayList<Boolean>();
           for (int v = 0; v < listobj1.size(); v++) {
            if (cocyc.get(u).contains(listobj1.get(v).pos)) line.add(false);
            else line.add(true);
           }
           matrix.add(line);
          }
          r2 = rank(matrix);
          for (int w = b; w < cocyc.size(); w++) {
           ArrayList<Boolean> line = new ArrayList<Boolean>();
           for (int v = 0; v < listobj1.size(); v++) {
            if (cocyc.get(w).contains(listobj1.get(v).pos)) line.add(false);
            else line.add(true);
           }
           matrix.add(line);
          }
          h1 = rank(matrix);
          for (int u = b-a-1; u >= 0; u--) matrix.remove(u);
          h2 = rank(matrix);
          r4 = h2+r2-h1;
      }
     }
     x1 = r2-r4;
     x2 = r1-r2-r3+r4;
     x3 = r4;
     x4 = r3-r4;
     System.out.println("( "+x1+", "+x2+", "+x3+", "+x4+")");
    }
    
    int rank(ArrayList<ArrayList<Boolean>> mat) {
    	int rnk = 0;
    	ArrayList<ArrayList<Boolean>> matrix = new ArrayList<ArrayList<Boolean>>();
    	for (ArrayList<Boolean> line : mat) {
    		ArrayList<Boolean> ln = new ArrayList<Boolean>();
    		for (Boolean bl : line) {
    			if (bl) ln.add(true);
    			else ln.add(false);
    		}
    		matrix.add(ln);
    	}
    	for (int u = 0; u < matrix.size(); u++) {
    		boolean found = false;
    		int v = 0;
    		while (!found & v < matrix.get(u).size()) {
    			if (!matrix.get(u).get(v)) found = true;
    			else v++;
    		}
    		if (found) {
    			rnk++;
    			boolean helper;
    			for (int e = 0; e < matrix.size(); e++) {
    				if (e != u) {
    					for (int f = matrix.get(0).size()-1; f >= v; f--) {
        					helper = !matrix.get(e).get(f);
        					if (!matrix.get(e).get(v) & !matrix.get(u).get(f)) matrix.get(e).set(f, helper);
        				}
    				}
    				
    			}
    		}
    	}
    	return rnk;
    }
    
    ArrayList<ArrayList<Integer>> obtainKerBock(ArrayList<ArrayList<Integer>> orcyc,
      ArrayList<ArrayList<Integer>> cocyc,ArrayList<m2obj> listobj0,
            ArrayList<m2obj> listobj1, ArrayList<m2obj> listobj2) {
     
     ArrayList<ArrayList<Integer>> kernel = new ArrayList<ArrayList<Integer>>();
     ArrayList<ArrayList<Boolean>> image = new ArrayList<ArrayList<Boolean>>();
     ArrayList<Integer> images = new ArrayList<Integer>();
     ArrayList<Boolean> line = new ArrayList<Boolean>();
     ArrayList<ArrayList<Integer>> cycles = Clone(cocyc);
     DetHomCls(cycles, listobj0,listobj1,listobj2);
     //cycles.get(0).add(204);cycles.get(0).add(213);//to obtain some different kernels
     for (int u = 0 ; u < cycles.size(); u++) {
      if (cycles.get(u).isEmpty()) kernel.add(orcyc.get(u));
      else images.add(u);
     }
     if (images.size() > 1) {
      for (int u : images) {
       line = new ArrayList<Boolean>();
          for (int v = 0; v < listobj1.size(); v++) {
           if (cycles.get(u).contains(listobj1.get(v).pos)) line.add(false);
           else line.add(true);
          }
          image.add(line);
         }
      ArrayList<ArrayList<Integer>> basis = new ArrayList<ArrayList<Integer>>();
      ArrayList<ArrayList<Integer>> toker = new ArrayList<ArrayList<Integer>>();
      for (int u : images) {
       ArrayList<Integer> base = new ArrayList<Integer>();
       base.add(u);
       basis.add(base);
      }
      for (int i = 0; i < image.size(); i++) {
       boolean found = false;
       int j = 0;
       while (!found & (j < image.get(i).size())) {
        if (!image.get(i).get(j)) found = true;
        else j++;
       }
       if (!found) toker.add(basis.get(i));
       else {
        int k = j+1;
        boolean helper;
        while (k < image.get(i).size()) {
         if (!image.get(i).get(k)) {
          for (int l = i; l < image.size(); l++) {
           if (!image.get(l).get(j)) {
            helper = !image.get(l).get(k);
            image.get(l).set(k,helper);
           }
          }
         }
         k++;
        }
        for (int l = i+1; l < image.size(); l++) {
         if (!image.get(l).get(j)) {
          helper = !image.get(l).get(j);
          image.get(l).set(j, helper);
          for (int y : basis.get(i)) {
           if (basis.get(l).contains(y)) 
            basis.get(l).remove((Integer)y);
           else basis.get(l).add(y);
          }
         }
        }
       }
      }
      for (ArrayList<Integer> tok : toker) {
       ArrayList<Integer> newker = new ArrayList<Integer>();
       for (int r : tok) {
        for (int e : orcyc.get(r)) {
         if (newker.contains(e)) newker.remove((Integer)e);
         else newker.add(e);
        }
       }
       kernel.add(newker);
      }
     }
     return kernel;
    }
    
    ArrayList<ArrayList<Integer>> Clone(ArrayList<ArrayList<Integer>> listint) {
     ArrayList<ArrayList<Integer>> clon = new ArrayList<ArrayList<Integer>>();
     ArrayList<Integer> clo;
     for (ArrayList<Integer> dude : listint) {
      clo = new ArrayList<Integer>();
      for (int dud : dude) clo.add(dud);
      clon.add(clo);
     }
     return clon;
    }
    
    void ShowHomCls(ArrayList<ArrayList<Integer>> cocyc, ArrayList<m2obj> listobj0,
            ArrayList<m2obj> listobj1, ArrayList<m2obj> listobj2) {
     DetHomCls(cocyc, listobj0,listobj1,listobj2);
        for (ArrayList<Integer> cyc : cocyc) {
            System.out.print("Represents : ");
            for (int e : cyc) System.out.print(e+" ");
            System.out.println();
        }
    }
    
    void DetHomCls(ArrayList<ArrayList<Integer>> cocyc, ArrayList<m2obj> listobj0,
            ArrayList<m2obj> listobj1, ArrayList<m2obj> listobj2) {
        int u = 0;
        int v;
        m2obj cand,y,w;
        while (u < listobj1.size()) {
            cand = listobj1.get(u);
            if (cand.above.isEmpty() & cand.below.isEmpty()) u++;
            else { // cancel cand
                if (cand.above.size() < cand.below.size()) { // cancel with a lower one
                    y = cand.below.get(0);         
                    v = 1;
                    for (ArrayList<Integer> cyc : cocyc) {
                        if (cyc.contains(cand.pos)) {
                            cyc.remove((Integer) cand.pos);
                            for (m2obj z : listobj1) {
                                if (z != cand & z.below.contains(y)) {
                                    if (cyc.contains(z.pos)) 
                                        cyc.remove((Integer) z.pos);
                                    else cyc.add(z.pos);
                                }
                            }
                        }
                        
                    }
                    while (v < cand.below.size()) {
                        w = cand.below.get(v);
                        for (m2obj z : y.above) {
                            if (z!=cand)
                                if (w.above.contains(z)) {
                                    w.above.remove(z);
                                    z.below.remove(w);
                                }
                                else {
                                    w.above.add(z);
                                    z.below.add(w);
                                }
                        }
                        v++;
                    }
                    for (m2obj z : cand.below) z.above.remove(cand);
                    for (m2obj z : cand.above) z.below.remove(cand);
                    for (m2obj z : y.above) z.below.remove(y);
                    listobj1.remove(cand);
                    listobj0.remove(y);
                }
                else { // cancel with a higher one
                    w = cand.above.get(0);
                    for (ArrayList<Integer> cyc : cocyc) {
                        if (cyc.contains((Integer) cand.pos)) 
                            cyc.remove((Integer) cand.pos);
                    }
                    v = 1;
                    while (v < cand.above.size()) {
                        y = cand.above.get(v);
                        for (m2obj z : w.below) {
                            if (z!=cand) 
                                if (y.below.contains(z)) {
                                    y.below.remove(z);
                                    z.above.remove(y);
                                }
                                else {
                                    y.below.add(z);
                                    z.above.add(y);
                                }
                        }
                        v++;
                    }
                    for (m2obj z : cand.above) z.below.remove(cand);
                    for (m2obj z : cand.below) z.above.remove(cand);
                    for (m2obj z : w.below) z.above.remove(w);
                    listobj1.remove(cand);
                    listobj2.remove(w);
                }
            }
        }
    }
    
    ArrayList<ArrayList<Integer>> Steenrod(ArrayList<ArrayList<Integer>> coccs,
            ArrayList<flow_object> obj1, ArrayList<flow_object> obj2,
            ArrayList<flow_object> obj3, link Link) {
     ArrayList<ArrayList<Integer>> steen = new ArrayList<ArrayList<Integer>>();
     ArrayList<vertex> vertices = new ArrayList<vertex>();
        ArrayList<tmatch> topMatches = new ArrayList<tmatch>();
        ArrayList<Integer> topObjects = new ArrayList<Integer>();
     for (ArrayList<Integer> cocyc : coccs) {
            ArrayList<Integer> rod = new ArrayList<Integer>();
            for (int u : cocyc) {
      for (mod0 bot : obj1.get(u).moduli0b) {
                    flow_object y = bot.top;
                    addTopMatch(bot,topMatches,obj2);
                    for (mod0 top : y.moduli0b) {
                        vertex v = new vertex(bot,top);
       vertices.add(v);
                        addTopObject(obj3.indexOf(top.top), topObjects);
                    }
      }
            }
            ArrayList<modedge> onedimmod = matchVertices(vertices,Link);
            ArrayList<matedge> tMatedges = tmatchVertices(vertices, topMatches);
            for (int e : topObjects) {
                int v = 0;
                int sum = 0;
                while (v < vertices.size()) {
                    if (!vertices.get(v).passed) {
                        if (vertices.get(v).top.top == obj3.get(e)) {
                            vertices.get(v).passed = true;
                            sum++;
                            boolean pass = false;
                            vertex current = vertices.get(v);
                            while (!pass) { // chase around the circle containing our current vertex
                                int pick = current.pairmod;
                                if (onedimmod.get(pick).a == current)
                                    current = onedimmod.get(pick).b;
                                else current = onedimmod.get(pick).a;
                                current.passed = true;
                                if (!onedimmod.get(pick).sign) sum++;
                                pick = current.pairmat;
                                if (tMatedges.get(pick).a == current) {
                                    current = tMatedges.get(pick).b;
                                    if (tMatedges.get(pick).directed) 
                                        if (tMatedges.get(pick).direction) sum++;
                                }
                                else {
                                    current = tMatedges.get(pick).a;
                                    if (tMatedges.get(pick).directed) 
                                        if (!tMatedges.get(pick).direction) sum++;
                                }
                                pass = current.passed;
                                current.passed = true;
                            }
                        }
                    }
                    v++;
                }
                if (sum%2 != 0) rod.add(e);
            }
            steen.add(rod);
            vertices.clear();
            topMatches.clear();
            topObjects.clear();
     }
     return steen;
    }
    
    ArrayList<matedge> tmatchVertices(ArrayList<vertex> vrtcs,
            ArrayList<tmatch> tmtchs) {
        ArrayList<matedge> edges = new ArrayList<matedge>();
        for (vertex vert : vrtcs) {
            if (vert.pairmat == -1) {
                int u = 0;
                boolean found = false;
                while (!found) {
                    if (tmtchs.get(u).mods.contains(vert.bot)) found = true;
                    else u++;
                }
                int v = tmtchs.get(u).mods.indexOf(vert.bot);
                if (v%2 == 0) v++;
                else v--;
                mod0 mod = tmtchs.get(u).mods.get(v);
                found = false;
                int s = vrtcs.indexOf(vert)+1;
                vertex help = vrtcs.get(s);
                while (!found) {
                    if ((help.bot == mod) & (help.top == vert.top)) found = true;
                    else {
                        s++;
                        help = vrtcs.get(s);
                    }
                }
                vert.pairmat = edges.size();
                help.pairmat = edges.size();
                boolean directed;
                boolean direction = true;
                if (vert.bot.sign == help.bot.sign) directed = true;
                else directed = false;
                if (directed) {
                    if (v%2==1) direction = true;
                    else direction = false;
                }
                matedge edge = new matedge(vert,help,directed,direction);
                edges.add(edge);
            }
        }
        return edges;
    }
    
    void addTopObject(int ob, ArrayList<Integer> objs) {
        if (objs.contains(ob)) return;
        objs.add(ob);
    }
    
    void addTopMatch(mod0 bot, ArrayList<tmatch> matches,
            ArrayList<flow_object> obj2) {
        boolean found = false;
        int number = obj2.indexOf(bot.top);
        int r = 0;
        while ((!found) & r < matches.size()) {
            if (matches.get(r).ob == number) found = true;
            else r++;
        }
        if (found) matches.get(r).mods.add(bot);
        else {
            tmatch match = new tmatch(number);
            match.mods.add(bot);
            matches.add(match);
        }
    }
    
    ArrayList<modedge> matchVertices(ArrayList<vertex> verts, link Link) {
     ArrayList<modedge> moders = new ArrayList<modedge>();
        int pos;
        int frame,frame2;
        int ladybird = -1;
        vertex w;
     for (vertex a : verts) {
            if (a.pairmod==-1) { //find the partner of a
                pos = verts.indexOf(a)+1; 
                w = verts.get(pos);
                frame = 0;
                int x = posDifference(a.bot);
                int y = posDifference(a.top);
      int a1 = a.bot.bot.dig.position[x];
      int a2 = a.top.bot.dig.position[y];
      int b1 = a.bot.which;
      int b2 = a.top.which;
                if (x!=y) ladybird = ladyBug(a.bot.bot,x,y,a1,a2,Link);
                boolean found = false;
                while (!found & pos<verts.size()) {
                    w = verts.get(pos);
                    if ((w.bot.bot == a.bot.bot) & (w.top.top == a.top.top) & 
                            (w.pairmod==-1)) {
                        if (x==y) { // it is an N or \tilde{N} cell
                            if (((a1 < 0) & (a1%2==0)) | ((a1 > 0) & (a1%2)!=0)) { // N cell
                                if (a2 == -1) {
                                    found = true;
                                    for (int r = 0; r < x; r++)
                                        frame = frame + a.bot.bot.dig.position[r];
                                }
                                else {
                                    int b3 = w.bot.which;
                                    int b4 = w.top.which;
                                    if (b1+b2 == b3+b4) {
                                        found = true;
                                        for (int r = 0; r < x; r++)
                                            frame = frame + a.bot.bot.dig.position[r];
                                    }
                                }
                            }
                            else { //\tilde{N} cell
                                if (a1 == 0) found = true;
                                else {
                                    int b3 = w.bot.which;
                                    int b4 = w.top.which;
                                    if (b1+b2 == b3+b4) found = true;
                                }
                            }
                        }
                        else { // it is an (E,E) cell
                            int xx = posDifference(w.top);
                            if (xx == x) {
                                int b3 = w.bot.which;
                                int b4 = w.top.which;
                                if ((b1 == b4) & (b2 == b3)) { // need to check not a ladybug!
                                    if (ladybird==-1) {
                                        found = true;
                                        boolean Mfirst = false,Msecond = false;
                                        if (((a1<0) & (a1%2 == 0))|(a1>0) & (a1%2!=0)) {
                                            if (b1 == 1) Mfirst = true;
                                            else Mfirst = false;
                                        }
                                        if (((a2<0) & (a2%2 == 0))|(a2>0) & (a2%2!=0)) {
                                            if (b2 == 1) Msecond = true;
                                            else Msecond = false;
                                        }
                                        frame2 = 0;
                                        if (x < y) {
                                            for (int r = 0; r < x; r++) 
                                                frame = frame + a.bot.bot.dig.position[r];
                                            if (Mfirst) {
                                                frame++;
                                                if (Link.crossings[x] < 0) frame2 = 1;
                                            }
                                            else frame2 = a1;
                                            if (Msecond) frame2++;
                                            for (int r = x+1; r < y; r++)
                                                frame2 = frame2 + a.bot.bot.dig.position[r];
                                            frame = frame * frame2;
                                        }
                                        else {
                                            for (int r = 0; r < y; r++)
                                                frame = frame + a.bot.bot.dig.position[r];
                                            if (Msecond) {
                                                frame++;
                                                if (Link.crossings[y] < 0) frame2 = 1;
                                            }
                                            else frame2 = a2;
                                            if (Mfirst) frame2++;
                                            for (int r = y+1; r < x; r++)
                                                frame2 = frame2 + a.bot.bot.dig.position[r];
                                            frame = frame * frame2;
                                        }
                                    }
                                    else { //ladybug!!!
                                        int [] cr1 = Link.paths[x].clone();
                                        int [] cr2 = Link.paths[y].clone();
                                        if (a1 == -1) {
                                            int help = cr1[0];
                                            cr1[0] = cr1[3];
                                            cr1[3] = cr1[2];
                                            cr1[2] = cr1[1];
                                            cr1[1] = help;
                                        }
                                        if (a2 == -1) {
                                            int help = cr2[0];
                                            cr2[0] = cr2[3];
                                            cr2[3] = cr2[2];
                                            cr2[2] = cr2[1];
                                            cr2[1] = help;
                                        }
                                        if (ladybird == 0) {
                                            int c1 = theComp(a.bot.top.dig.components,cr2[2]);
                                            int c2 = theComp(w.bot.top.dig.components,cr1[0]);
                                            if (a.bot.top.symbols.get(c1) == w.bot.top.symbols.get(c2))
                                                found = true;
                                        }
                                        else {
                                            int c1 = theComp(a.bot.top.dig.components,cr2[2]);
                                            int c2 = theComp(w.bot.top.dig.components,cr1[2]);
                                            if (a.bot.top.symbols.get(c1) == w.bot.top.symbols.get(c2))
                                                found = true;
                                        }
                                        if (found) {
                                            frame = 0;
                                            frame2 = 0;
                                            if (x < y) {
                                                for (int r = 0; r < x; r++) 
                                                    frame = frame + a.bot.bot.dig.position[r];
                                                for (int r = x; r < y; r++)
                                                    frame2 = frame2 + a.bot.bot.dig.position[r];
                                            }
                                            else {
                                                for (int r = 0; r < y; r++)
                                                    frame = frame + a.bot.bot.dig.position[r];
                                                for (int r = y; r < x; r++)
                                                    frame2 = frame2 + a.bot.bot.dig.position[r];
                                            }
                                            frame = frame * frame2;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    pos++;
                }
                pos--;
                a.pairmod = moders.size();
                w.pairmod = moders.size();
                boolean sign = (frame%2==0);
      modedge edge = new modedge(a,w,sign);
                moders.add(edge);
            }
        }
     return moders;
    }
    
    int ladyBug(flow_object ob, int x, int y, int a, int b, link Link) {
        int bug = -1;
        if ((a < -1) | (b < -1) | (a > 0) | (b > 0)) return bug;
        int [] cr1 = Link.paths[x].clone();
        int [] cr2 = Link.paths[y].clone();
        if (a == -1) {
            int help = cr1[0];
            cr1[0] = cr1[3];
            cr1[3] = cr1[2];
            cr1[2] = cr1[1];
            cr1[1] = help;
        }
        if (b == -1) {
            int help = cr2[0];
            cr2[0] = cr2[3];
            cr2[3] = cr2[2];
            cr2[2] = cr2[1];
            cr2[1] = help;
        }
        int ch1 = theComp(ob.dig.components,cr1[0]);
        int ch2 = theComp(ob.dig.components,cr1[2]);
        int ch3 = theComp(ob.dig.components,cr2[0]);
        int ch4 = theComp(ob.dig.components,cr2[2]);
        if ((ch1 != ch2) | (ch1 != ch3) | (ch1 != ch4)) return bug;
        ArrayList<Integer> comp = ob.dig.components.get(ch1);
        int posa = comp.indexOf(cr1[0]);
        int posb = comp.indexOf(cr1[1]);
        boolean inc;
        if ((posa - posb == 1) | (posa == 0 & posb == comp.size()-1)) inc = true;
        else inc = false;
        boolean found = false;
        while (!found) {
            if ((comp.get(posa) == cr2[0])|(comp.get(posa) == cr2[2]))
                found = true;
            else {
                if (comp.get(posa) == cr1[3]) return bug;
                if (inc) posa++;
                else posa--;
                if (posa == comp.size()) posa = 0;
                if (posa == -1) posa = comp.size()-1;
            }
        }
        if (comp.get(posa) == cr2[0]) {
            if (inc) posa++;
            else posa--;
            if (posa == comp.size()) posa = 0;
            if (posa == -1) posa = comp.size()-1;
            if (comp.get(posa) == cr2[1]) bug = 0;
        }
        else {
            if (inc) posa++;
            else posa--;
            if (posa == comp.size()) posa = 0;
            if (posa == -1) posa = comp.size()-1;
            if (comp.get(posa) == cr2[3]) bug = 1;
        }
        if (bug == 0) { // bug == 0 means b>a>e>f>d>c>g>h>b
            found = false;
            while (!found) {
                if ((comp.get(posa) == cr2[2]) | (comp.get(posa) == cr1[3]))
                    found = true;
                else {
                    if (inc) posa++;
                    else posa--;
                    if (posa == comp.size()) posa = 0;
                    if (posa == -1) posa = comp.size()-1;
                }
            }
            if (comp.get(posa) == cr2[2]) bug = -1;
        }
        if (bug == 1) { // bug == 1 means b>a>g>h>d>c>e>f>b
            found = false;
            while (!found) {
                if ((comp.get(posa) == cr2[0]) | (comp.get(posa) == cr1[3]))
                    found = true;
                else {
                    if (inc) posa++;
                    else posa--;
                    if (posa == comp.size()) posa = 0;
                    if (posa == -1) posa = comp.size()-1;
                }
            }
            if (comp.get(posa) == cr2[0]) bug = -1;
        }
        return bug;
    }
    
    int posDifference(mod0 mod) {
     flow_object top = mod.top;
     flow_object bot = mod.bot;
     boolean found = false;
     int e = 0;
     while (!found) {
            if (top.dig.position[e] == bot.dig.position[e]) e++;
            else found = true;
     }
     return e;
    }
    
    ArrayList<ArrayList<Integer>> Bockstein(ArrayList<ArrayList<Integer>> coccs,
            ArrayList<flow_object> obj1, ArrayList<flow_object> obj2) {
        ArrayList<ArrayList<Integer>> bock = new ArrayList<ArrayList<Integer>>();
        flow_object a;
        int x;
        int h;
        boolean value;
        for (ArrayList<Integer> cocycle : coccs) {
            ArrayList<Integer> helpers = new ArrayList<Integer>();
            for (int v = 0; v < obj2.size(); v++) {
                helpers.add(0);
            }
            for (int u : cocycle) {
                for (int w = 0; w < obj1.get(u).moduli0b.size(); w++) {
                    a = obj1.get(u).moduli0b.get(w).top;
                    value = obj1.get(u).moduli0b.get(w).sign;
                    x = obj2.indexOf(a);
                    h = helpers.get(x);
                    if (value) h++;
                    else h--;
                    helpers.set(x,h);
                }
            }
            ArrayList<Integer> bck = new ArrayList<Integer>();
            for (int v = 0; v < obj2.size(); v++) 
                if (((helpers.get(v)/2)%2)!=0) bck.add(v);
            bock.add(bck);
        }
        return bock;
    }
    
    ArrayList<ArrayList<Integer>> getCocycles(ArrayList<m2obj> listobj0,
            ArrayList<m2obj> listobj1, ArrayList<m2obj> listobj2) {
        ArrayList<ArrayList<Integer>> cocycles = new ArrayList<ArrayList<Integer>>();
        ArrayList<ArrayList<Integer>> cancelList = new ArrayList<ArrayList<Integer>>();
        int u = 0;
        int v;
        m2obj cand,y,w;
        while (u < listobj1.size()) {
            cand = listobj1.get(u);
            if (cand.above.isEmpty() & cand.below.isEmpty()) u++;
            else { // cancel cand
                if (cand.above.size() < cand.below.size()) { // cancel with a lower one
                    y = cand.below.get(0);         
                    v = 1;
                    while (v < cand.below.size()) {
                        w = cand.below.get(v);
                        for (m2obj z : y.above) {
                            if (z!=cand)
                                if (w.above.contains(z)) {
                                    w.above.remove(z);
                                    z.below.remove(w);
                                }
                                else {
                                    w.above.add(z);
                                    z.below.add(w);
                                }
                        }
                        v++;
                    }
                    for (m2obj z : cand.above) z.below.remove(cand);
                    for (m2obj z : y.above) z.below.remove(y);
                    listobj1.remove(cand);
                    listobj0.remove(y);
                }
                else { // cancel with a higher one
                    ArrayList<Integer> helper = new ArrayList<Integer>();
                    helper.add(cand.pos);
                    w = cand.above.get(0);// cancel cand with w
                    for (m2obj z : w.below) if (z!=cand) helper.add(z.pos);
                    cancelList.add(helper); // keep track of cancellation
                    v = 1;
                    while (v < cand.above.size()) {
                        y = cand.above.get(v);
                        for (m2obj z : w.below) {
                            if (z!=cand)
                                if (y.below.contains(z)) {
                                    y.below.remove(z);
                                    z.above.remove(y);
                                }
                                else {
                                    y.below.add(z);
                                    z.above.add(y);
                                }
                        }
                        v++;
                    }
                    for (m2obj z : cand.below) z.above.remove(cand);
                    for (m2obj z : w.below) z.above.remove(w);
                    listobj1.remove(cand);
                    listobj2.remove(w);
                }
            }
        }// now recover the cocycles
        for (m2obj z : listobj1) {
            ArrayList<Integer> cocycle = new ArrayList<Integer>();
            cocycle.add(z.pos);
            cocycles.add(cocycle);
        }
        int g = cancelList.size()-1;
        ArrayList<Boolean> checker = new ArrayList<Boolean>();
        boolean check;
        while (g >= 0) {
            ArrayList<Integer> help = cancelList.get(g);
            int cancelled = help.get(0);
            int h = 1;
            int j;
            checker.clear();
            for (int a = 0; a < cocycles.size(); a++) checker.add(true);
            while (h < help.size()) {
                j = help.get(h);
                for (int b = 0; b < cocycles.size(); b++) 
                    if (cocycles.get(b).contains(j)) {
                        check = !checker.get(b);
                        checker.set(b,check);
                    }
                h++;
            }
            for (int a = 0; a < checker.size(); a++) 
                if (!checker.get(a)) cocycles.get(a).add(cancelled);
            g--;
        }
        return cocycles;
    }
    
    final void AllMod2Hom(link Link, int n) {
        
        int qmax = maxQ(Link,n);
        int qmin = minQ(Link,n);
        int hmax = maxH(Link,n);
        int hmin = minH(Link,n);
        int qdeg = qmax;
        int hdeg;
        
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        while (qdeg >= qmin) {
            hdeg = hmin;
            first = true;
            objects0 = getObjects(Link, hdeg-1, qdeg, n);
            objects1 = getObjects(Link, hdeg, qdeg, n);
            objects2 = getObjects(Link, hdeg+1, qdeg, n);
            get0Mod(objects1,objects0,Link.paths,n);
            get0Mod(objects2,objects1,Link.paths,n);
            ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
            ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
            ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
            for (int u = 0; u < objects0.size(); u++) {
                m2obj help = new m2obj(u);
                listobj0.add(help);
            }
            for (int u = 0; u < objects1.size(); u++) {
                m2obj help = new m2obj(u);
                listobj1.add(help);
            }
            for (int u = 0; u < objects2.size(); u++) {
                m2obj help = new m2obj(u);
                listobj2.add(help);
            }
            createM2mods(objects1,objects0,listobj1,listobj0);
            createM2mods(objects2,objects1,listobj2,listobj1);
            calcMod2Hom(listobj0,listobj1,listobj2,hdeg,qdeg,n,false);
            hdeg++;
            while (hdeg <= hmax) {
                objects0 = objects1;
                objects1 = objects2;
                objects2 = getObjects(Link, hdeg+1, qdeg, n);
                get0Mod(objects2,objects1,Link.paths,n);
                listobj0.clear();
                listobj1.clear();
                listobj2.clear();
                for (int u = 0; u < objects0.size(); u++) {
                    m2obj help = new m2obj(u);
                    listobj0.add(help);
                }
                for (int u = 0; u < objects1.size(); u++) {
                    m2obj help = new m2obj(u);
                    listobj1.add(help);
                }
                for (int u = 0; u < objects2.size(); u++) {
                    m2obj help = new m2obj(u);
                    listobj2.add(help);
                }
                createM2mods(objects1,objects0,listobj1,listobj0);
                createM2mods(objects2,objects1,listobj2,listobj1);
                calcMod2Hom(listobj0,listobj1,listobj2,hdeg,qdeg,n,false);
                hdeg++;
            }
            if (!first) System.out.println(" ) q^"+qdeg);
            qdeg= qdeg-2;
        }
    }
    
    int maxQ(link Link, int n) {
        int t = 0;
        if (n==2) t = sockflow.size()-1;
        diagram dig = sockflow.get(t);
        int q = qandeg(dig, Link, n);
        q = q + (n-1)*dig.components.size();
        return q;
    }
    
    int minQ(link Link, int n) {
        int t = sockflow.size()-1;
        if (n==2) t = 0;
        diagram dig = sockflow.get(t);
        int q = qandeg(dig,Link, n);
        q = q - (n-1)*dig.components.size();
        return q;
    }
    
    int maxH(link Link, int n) {
        diagram dig = sockflow.get(sockflow.size()-1);
        int h = homdeg(dig, Link, n);
        return h;
    }
    
    int minH(link Link, int n) {
        diagram dig = sockflow.get(0);
        int h = homdeg(dig,Link, n);
        return h;
    }
    
    final void Mod2Homology(link Link, int hdeg, int qdeg, int n) {
        ArrayList<flow_object> objects0;
        ArrayList<flow_object> objects1;
        ArrayList<flow_object> objects2;
        objects0 = getObjects(Link, hdeg-1, qdeg, n);
        objects1 = getObjects(Link, hdeg, qdeg, n);
        objects2 = getObjects(Link, hdeg+1, qdeg, n);
        get0Mod(objects1,objects0,Link.paths,n);
        get0Mod(objects2,objects1,Link.paths,n);
        ArrayList<m2obj> listobj0 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj1 = new ArrayList<m2obj>();
        ArrayList<m2obj> listobj2 = new ArrayList<m2obj>();
        for (int u = 0; u < objects0.size(); u++) {
            m2obj help = new m2obj(u);
            listobj0.add(help);
        }
        for (int u = 0; u < objects1.size(); u++) {
            m2obj help = new m2obj(u);
            listobj1.add(help);
        }
        for (int u = 0; u < objects2.size(); u++) {
            m2obj help = new m2obj(u);
            listobj2.add(help);
        }
        createM2mods(objects1,objects0,listobj1,listobj0);
        createM2mods(objects2,objects1,listobj2,listobj1);
        calcMod2Hom(listobj0,listobj1,listobj2,hdeg,qdeg,n,true);
    }
    
    void calcMod2Hom(ArrayList<m2obj> listobj0,ArrayList<m2obj> listobj1,
            ArrayList<m2obj> listobj2, int hdeg, int qdeg, int n, 
            boolean show) {
        int u = 0;
        int v;
        m2obj cand,y,w;
        while (u < listobj1.size()) {
            cand = listobj1.get(u);
            if (cand.above.isEmpty() & cand.below.isEmpty()) u++;
            else { // cancel cand
                if (cand.above.size() < cand.below.size()) { // cancel with a lower one
                    y = cand.below.get(0);         
                    v = 1;
                    while (v < cand.below.size()) {
                        w = cand.below.get(v);
                        for (m2obj z : y.above) {
                            if (z!=cand)
                                if (w.above.contains(z)) {
                                    w.above.remove(z);
                                    z.below.remove(w);
                                }
                                else {
                                    w.above.add(z);
                                    z.below.add(w);
                                }
                        }
                        v++;
                    }
                    for (m2obj z : cand.below) z.above.remove(cand);
                    for (m2obj z : cand.above) z.below.remove(cand);
                    for (m2obj z : y.above) z.below.remove(y);
                    listobj1.remove(cand);
                    listobj0.remove(y);
                }
                else { // cancel with a higher one
                    w = cand.above.get(0);
                    v = 1;
                    while (v < cand.above.size()) {
                        y = cand.above.get(v);
                        for (m2obj z : w.below) {
                            if (z!=cand) 
                                if (y.below.contains(z)) {
                                    y.below.remove(z);
                                    z.above.remove(y);
                                }
                                else {
                                    y.below.add(z);
                                    z.above.add(y);
                                }
                        }
                        v++;
                    }
                    for (m2obj z : cand.above) z.below.remove(cand);
                    for (m2obj z : cand.below) z.above.remove(cand);
                    for (m2obj z : w.below) z.above.remove(w);
                    listobj1.remove(cand);
                    listobj2.remove(w);
                }
            }
        }
        int betti = listobj1.size();
        if ((!show) & (betti == 0) ) return;
        if (show) {
         System.out.print("sl("+n+") ");
         System.out.println("H("+hdeg+","+qdeg+")");
            System.out.println("Z/2-Betti : "+betti);
        }
        else {
         if (first) System.out.print("( ");
         else System.out.print(" + ");
         if (betti > 1) System.out.print(betti+" ");
         System.out.print("t^"+hdeg);
         first = false;
        }
    }
    
    void createM2mods(ArrayList<flow_object> obs1, ArrayList<flow_object> obs0,
            ArrayList<m2obj> list1, ArrayList<m2obj> list0) {
        int v;
        for (int u = 0; u < obs1.size(); u++) {
            ArrayList<Integer> helpers = new ArrayList<Integer>();
            for (mod0 mod : obs1.get(u).moduli0t) {
                v = obs0.indexOf(mod.bot);
                if (helpers.contains((Integer)v)) helpers.remove((Integer) v);
                else helpers.add(v);
            }
            for (int x : helpers) {
                list1.get(u).below.add(list0.get(x));
                list0.get(x).above.add(list1.get(u));
            }
        }
    }
    
    link Mirror(link Link) {
        int [] mcross = new int[Link.crossings.length];
        for (int u=0; u < Link.crossings.length; u++) 
            mcross[u] = -Link.crossings[u];
        link mirror = new link(mcross, Link.paths);
        return mirror;
    }
    
    void get0Mod(ArrayList<flow_object> obj1, ArrayList<flow_object> obj0,
            int[] [] paths, int n) {
        int pos;
        int a,b,sgn,c,d,f,g;
        boolean sign = true;
        for (flow_object ob1 : obj1) {
            for (flow_object ob0 : obj0) {
                pos = posCheck(ob1,ob0);
                if (pos >= 0) {// ob0 could be related to ob1
                    a = ob1.dig.position[pos];
                    b = ob0.dig.position[pos];
                    if (a == 0 | b == 0) { // potential Surgery!
                        if (otherCirclesSame(ob1,ob0,paths[pos])) { // now surgery
                            sgn = 0; 
                            for (int t=0; t < pos; t++) 
                                sgn = sgn + ob0.dig.position[t];
                            if (sgn % 2 == 0) sign = true;
                            else sign = false;
                            mod0 mod = new mod0(sign,0,ob1,ob0);
                            ob1.moduli0t.add(mod);
                            ob0.moduli0b.add(mod);
                        }
                    }
                    else { //potential Non-Surgery!!
                        if (otherCirclesSame(ob1,ob0,paths[pos])) { // now non-surgery
                            c = theComp(ob1.dig.components,paths[pos][0]);
                            d = theComp(ob1.dig.components,paths[pos][1]);
                            sgn = 0;
                            for (int t=0; t < pos; t++) 
                                sgn = sgn + ob0.dig.position[t];
                            if (b < 0) f = a;
                            else f = b;
                            if (sgn % 2 == 0) sign = true;
                            else sign = false;
                            if (c == d) {
                                if (f%2 == 0) { //add n points
                                    for (int e = 0; e < n; e++) {
                                        mod0 mod = new mod0(sign,e,ob1,ob0);
                                        ob1.moduli0t.add(mod);
                                        ob0.moduli0b.add(mod);
                                    }
                                }
                                else { // add two points of different sign
                                    mod0 mod = new mod0(sign,0,ob1,ob0);
                                    ob1.moduli0t.add(mod);
                                    ob0.moduli0b.add(mod);
                                    sign = !sign;
                                    mod = new mod0(sign,1,ob1,ob0);
                                    ob1.moduli0t.add(mod);
                                    ob0.moduli0b.add(mod);
                                }
                            }
                            else { //different circles
                                if (f%2 == 0) { // one of the n points
                                 g = ((int) ob1.symbols.get(c))-((int) ob0.symbols.get(c));
                                    if (n == 2) g = -g;
                                 if (g < 0) System.out.println(g+"Warning!");
                                 mod0 mod = new mod0(sign,g,ob1,ob0);
                                    ob1.moduli0t.add(mod);
                                    ob0.moduli0b.add(mod);
                                }
                                else { // plus or minus
                                    if (ob1.symbols.get(c)==ob0.symbols.get(c)) {
                                        mod0 mod = new mod0(sign,0,ob1,ob0);
                                        ob1.moduli0t.add(mod);
                                        ob0.moduli0b.add(mod);
                                    }
                                    if (ob1.symbols.get(d)==ob0.symbols.get(d)) {
                                        sign = !sign;
                                        mod0 mod = new mod0(sign,1,ob1,ob0);
                                        ob1.moduli0t.add(mod);
                                        ob0.moduli0b.add(mod);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    boolean otherCirclesSame(flow_object ob1, flow_object ob0, int [] path) {
        boolean ok = true;
        int u = 0;
        int pos;
        while (ok & (u < ob1.symbols.size())) {
            ArrayList<Integer> comp = ob1.dig.components.get(u);
            if ((!comp.contains(path[0]))&(!comp.contains(path[1]))&
                    (!comp.contains(path[2]))&(!comp.contains(path[3]))) {
                pos = theComp(ob0.dig.components,comp.get(0));
                if (ob1.symbols.get(u)!=ob0.symbols.get(pos)) ok = false;
            }
            u++;
        }
        return ok;
    }
    
    int theComp(ArrayList<ArrayList<Integer>> comps, int c) {   // Finds the component among
        boolean found = false;         // comps which contains c
        int u = 0;
        ArrayList<Integer> comp;
        while (!found) {
            comp = comps.get(u);
            int v = 0;
            while (!found & v < comp.size()) {
                if (c == comp.get(v)) found = true;
                else v++;
            }
            if (!found) u++;
        }
        return u;
    }
    
    int posCheck(flow_object ob1, flow_object ob0) {
        int r = -1;
        int df = 0;
        int u = 0;
        int a,b;
        while (u < ob1.dig.position.length) {
            a = ob1.dig.position[u];
            b = ob0.dig.position[u];
            if ((a-b)<0 & (a-b)>1) {
                r = -1;
                break;
            }
            else {
                if (a==b) u++;
                else {
                    r = u;
                    df++;
                    u++;
                }
            }
            if (df > 1) break;
        }
        if (df > 1) r = -1;
        return r;
    }
    
    final ArrayList<flow_object> getObjects(link Link, int hdeg, int qdeg, 
            int n) {
        ArrayList<flow_object> objects = new ArrayList<flow_object>();
        if (n==2) {
            for (diagram dig : sockflow) {
                if (homdeg(dig, Link,2) == hdeg ) {
                    int q = qandeg(dig,Link,2)-qdeg; 
                    int r = dig.components.size();
                    if ((q+r)% 2 == 0) {
                        if ( (q>=-r) & (q <=r) ) { //diagram is ok
                            for (byte u = 0; u < 2; u++) {
                                ArrayList<Byte> symb = new ArrayList<Byte>();
                                shufflesigns(dig,symb,u,q,objects,2);
                            }
                        }
                    }
                }
            }
        }
        else {
            for (diagram dig : sockflow) {
                if (homdeg(dig, Link,n) == (hdeg)) {
                    int q = qandeg(dig,Link,n)-qdeg;
                    int r = dig.components.size();
                    if ((q >= -(n-1)*r) & (q <= (n-1)*r)) {
                        for (byte u = 0; u < n; u++) {
                            ArrayList<Byte> symb = new ArrayList<Byte>();
                            shufflesigns(dig,symb,u,q,objects,n);
                        }
                    }
                }
            }
        }
        return objects;
    }
    
    void shufflesigns(diagram dig, ArrayList<Byte> symb, byte u, int q, 
            ArrayList<flow_object> objects, int n) {
        ArrayList<Byte> signs = (ArrayList<Byte>) symb.clone();
        signs.add(u);
        q = q + (1-n) + 2*u;
        int r = dig.components.size()-signs.size();
        if ( (q >= -r*(n-1)) & ( q <= r*(n-1))) {
            if (r==0) {
                flow_object flobj = new flow_object(dig,signs);
                objects.add(flobj);
                return;
            }
            for (byte v = 0; v < n; v++)
                shufflesigns(dig,signs,v,q,objects,n);
        }
    }
    
    int qandeg(diagram dig, link Link, int n) {
        int q = 0;
        int k,m;
        if (n==2) {
            q = homdeg(dig,Link,2) + nplus(Link) - nminus(Link);
            for (int u = 0; u< Link.crossings.length; u++) {
                if (dig.position[u] > 1) q = q + dig.position[u] - 1;
                if (dig.position[u] < -1) q = q + dig.position[u] + 1;
            }//*/
        }
        if (n > 2) {
            for (int u = 0; u < Link.crossings.length; u++) {
                m = dig.position[u];
                k = Link.crossings[u];
                if (k > 0) {
                    if (m == 0) q = q + k*n;
                    else q = q + n - 1 + 2 * (((k-m)+1)/2) + 2 * (n-1) * ((k-m)/2);
                }
                if (k < 0) {
                    if (m == 0) q = q + k * n;
                    else q = q - n + 1 + 2 * (((k-m)-1)/2) + 2 * (n-1) * ((k-m)/2);
                }
            }
        }
        return q;
    }
    
    int homdeg(diagram dig, link Link, int n) {
        int h = 0;
        if (n==2) {
            for (int u=0 ; u < Link.crossings.length; u++) {
                h = h + dig.position[u];
                if (Link.crossings[u] < 0) h = h - Link.crossings[u];
            }
            h = h - nminus(Link);
        }
        else {
            for (int u=0 ; u < Link.crossings.length; u++) {
                h = h + dig.position[u] - Link.crossings[u];
            }
        }
        return h;
    }
    
    final void ObtainSock(link Link) {
        int m = Link.crossings.length;
        layer Slayer = new layer(Link.crossings, null, Link.paths, Link);
    }
    
    final void PrintDiagram(diagram dig) {
        for (int u = 0; u < dig.position.length; u++) 
            System.out.print(dig.position[u]+" ");
        System.out.print(" --- ");
        for (int u = 0; u < dig.components.size(); u++) {
            System.out.print("[");
            for (int v = 0; v < dig.components.get(u).size(); v++)
                System.out.print(dig.components.get(u).get(v)+" ");
            System.out.print("] ");
        }
        System.out.println();
    }
    
    final void PrintObject(flow_object obj) {
        for (int u = 0; u < obj.symbols.size(); u++) {
            System.out.print(obj.symbols.get(u)+" ");
        }
        System.out.print(" | ");
        PrintDiagram(obj.dig);
    }
    
    int nminus(link Link) {
        return nplus(Mirror(Link));
    }
    
    int nplus(link Link) {
        int n = 0;
        int a,b,c,d;
        int co1,co2;
        int in1,in2,in3,in4;
        boolean rup = true;
        boolean lup = true;
        // int [][] path = Link.paths.clone();
        ArrayList<Integer> used = new ArrayList<Integer>();
        for (int r = 0; r < Link.crossings.length; r++) {
            a = Link.paths[r][0];
            b = Link.paths[r][1];
            c = Link.paths[r][2];
            d = Link.paths[r][3];
            if (Link.crossings[r]%2 == 0) {
                co1=0;
                co2=0;
                while (true) {
                    if (Link.components.get(co1).contains(a)) break;
                    co1++;
                }
                in1 = Link.components.get(co1).indexOf(a);
                in2 = Link.components.get(co1).indexOf(b);
                if ((in1 == 0) | (in2 == 0)) {
                    if (in1 == 0) {
                        if (!used.contains(a)) {
                            used.add(a);
                            rup = true;
                        }
                        else rup = false;
                    }
                    else {
                        if (!used.contains(b)) {
                            used.add(b);
                            rup = false;
                        }
                        else rup = true;
                    }
                }
                else {
                    if (in1 < in2) rup = true;
                    else rup = false;
                }
                while (true) {
                    if (Link.components.get(co2).contains(c)) break;
                    co2++;
                }
                in3 = Link.components.get(co2).indexOf(c);
                in4 = Link.components.get(co2).indexOf(d);
                if ((in3 == 0) | (in4 == 0)) {
                    if (in3 == 0) {
                        if (!used.contains(c)) {
                            used.add(c);
                            lup = false;
                        }
                        else lup = true;
                    }
                    else {
                        if (!used.contains(d)) {
                            used.add(d);
                            lup = true;
                        }
                        else lup = false;
                    }
                }
                else {
                    if (in3 < in4) lup = false;
                    else lup = true;
                }
                int factor = 1;
                if (!rup) factor = (-1);
                if (!lup) factor = factor * (-1);
                if (factor * Link.crossings[r] > 0) n = n + factor * 
                    Link.crossings[r];
            }
            else {
                co1=0;
                co2=0;
                while (true) {
                    if (Link.components.get(co1).contains(a)) break;
                    co1++;
                }
                in1 = Link.components.get(co1).indexOf(a);
                in2 = Link.components.get(co1).indexOf(c);
                if ((in1 == 0) | (in2 == 0)) {
                    if (in1 == 0) {
                        if (!used.contains(a)) {
                            used.add(a);
                            rup = true;
                        }
                        else rup = false;
                    }
                    else {
                        if (!used.contains(c)) {
                            used.add(c);
                            rup = false;
                        }
                        else rup = true;
                    }
                }
                else {
                    if (in1 < in2) rup = true;
                    else rup = false;
                }
                while (true) {
                    if (Link.components.get(co2).contains(b)) break;
                    co2++;
                }
                in3 = Link.components.get(co2).indexOf(b);
                in4 = Link.components.get(co2).indexOf(d);
                if ((in3 == 0) | (in4 == 0)) {
                    if (in3 == 0) {
                        if (!used.contains(b)) {
                            used.add(b);
                            lup = false;
                        }
                        else lup = true;
                    }
                    else {
                        if (!used.contains(d)) {
                            used.add(d);
                            lup = true;
                        }
                        else lup = false;
                    }
                }
                else {
                    if (in3 < in4) lup = false;
                    else lup = true;
                }
                int factor = 1;
                if (!rup) factor = (-1);
                if (!lup) factor = factor * (-1);
                if (factor * Link.crossings[r] > 0) n = n + factor * 
                    Link.crossings[r];
            }
        }
        return n;
    }
    
    private class layer{
        int [] crossings;
        int [] position;
        
        layer(int [] crossins, int [] pos, int [] [] path, link Link) {
            crossings = crossins;
            int [] newcross = new int[crossings.length];
            
            System.arraycopy(crossings, 0, newcross, 0, crossings.length);
            if (pos == null) {
                position = null;
                int [] newpos = new int[1];
                newcross[0] = 0;
                if (crossings[0] < 0) {
                    for (int u = crossings[0]; u <= 0; u++) {
                        newpos[0] = u;
                        layer lay_1 = new layer(newcross, newpos, path,Link);
                    }
                }
                else {
                    for (int u = 0; u <= crossings[0]; u++) {
                        newpos[0] = u;
                        layer lay1 = new layer(newcross, newpos, path,Link);
                    }
                }
            }
            else {
                position = new int[pos.length];
                System.arraycopy(pos, 0, position, 0, pos.length);
                int j = pos.length; 
                if (j == crossings.length) { // arrived, add to sock
                    int [][] altpaths = new int[path.length][4];
                    for (int i = 0; i < j ; i++) {
                            if ( position[i] == 0) altpaths[i]=path[i];
                            else {
                                altpaths[i][0] = path[i][3];
                                altpaths[i][1] = path[i][0];
                                altpaths[i][2] = path[i][1];
                                altpaths[i][3] = path[i][2];
                            }
                    }
                    diagram dig = new diagram(position, altpaths);
                    sockflow.add(dig);
                }
                else {
                    int [] newpos = new int[j+1];
                    System.arraycopy(pos, 0, newpos, 0, pos.length);
                    newcross[j] = 0;
                    if (crossings[j] < 0) {
                       for (int u = crossings[j]; u <= 0; u++) {
                          newpos[j] = u;
                          layer lay_1 = new layer(newcross, newpos, path,Link);
                        }
                    }
                    else {
                        for (int u = 0; u <= crossings[j]; u++) {
                            newpos[j] = u;
                            layer lay1 = new layer(newcross, newpos, path,Link);
                        }
                    }
                }   
            }
            
            
        }
    }
    
    private class diagram {
        int [] position;
        ArrayList<ArrayList<Integer>> components;
        
        diagram(int[] pos, int [] [] path) {
            position = pos;
            //paths = path;
            int[] nuller = new int[position.length];
            for (int u = 0; u < nuller.length; u++) nuller[u] = 0;
            link help = new link(nuller,path);
            components = help.components;
        }
    }

    private class flow_object {
        diagram dig;
        ArrayList<Byte> symbols;
        ArrayList<mod0> moduli0t,moduli0b;
        
        flow_object(diagram dg, ArrayList<Byte> symbs) {
            dig = dg;
            symbols = symbs;
            moduli0t = new ArrayList<mod0>();
            moduli0b = new ArrayList<mod0>();
        }
    }
    
    private class m2obj {
        int pos;
        ArrayList<m2obj> below;
        ArrayList<m2obj> above;
        
        m2obj(int p) {
            pos = p;
            below = new ArrayList<m2obj>();
            above = new ArrayList<m2obj>();
        }
    }
    
    private class mod0 {
        boolean sign;
        flow_object top;
        flow_object bot;
        byte which;
        
        mod0(boolean sgn, int whch, flow_object t, flow_object b) {
            sign = sgn;
            top = t;
            bot = b;
            which = (byte) whch;
        }
    }
    
    private class modedge {
     vertex a;
     vertex b;
     boolean sign;
     
     modedge(vertex one, vertex two, boolean sgn) {
      a = one;
      b = two;
      sign = sgn;
     }
    }
    
    private class matedge {
        vertex a;
        vertex b;
        boolean directed;
        boolean direction;
        
        matedge(vertex one, vertex two, boolean dired, boolean dirion) {
            a = one;
            b = two;
            directed = dired;
            direction = dirion;
        }
    }
    
    private class vertex {
     mod0 bot;
     mod0 top;
     int pairmod; //whether paired with another vertex for a moduli space and where to find it in the mod list
     int pairmat; //whether paired with another vertex from topological matching and where to find it in the mat list
     boolean passed; // knows whether vertex has been passed in the final circle
        
     vertex(mod0 b, mod0 t) {
      bot = b;
      top = t;
      pairmod = -1;// -1 means, not paired yet.
      pairmat = -1;// dito
                passed = false;
     }
    }
    
    private class tmatch {
        int ob;
        ArrayList<mod0> mods;
        
        tmatch(int o) {
            ob = o;
            mods = new ArrayList<mod0>();
        }
    }
    
    private class link {
        int [] crossings;
        int [] [] paths;
        ArrayList<ArrayList<Integer>> components;
        link(int [] crossins, int [] [] path) {
            crossings = crossins;
            paths = path;
            components = components(crossins,path);
        }
        
        final ArrayList<ArrayList<Integer>> components(int [] crossings,
                int [] [] paths) {
            ArrayList<ArrayList<Integer>> compts = new ArrayList<ArrayList<Integer>>();
            boolean[] notcheckedpaths = new boolean[2*crossings.length];
            for (int u = 0; u < notcheckedpaths.length; u++) 
                notcheckedpaths[u] = true;
            int path = 1;
            int endpath, runpath;
            int pos = 0;
            int [] [] pafs = new int [paths.length][paths[0].length];
            for (int u = 0;u < pafs.length; u++) {
                System.arraycopy(paths[u], 0, pafs[u], 0, pafs[0].length);
            }
            while (path <= 2 * crossings.length) {
                ArrayList<Integer> comp = new ArrayList<Integer>();
                comp.add(path);
                notcheckedpaths[path - 1] = false;
                boolean comprun = true;
                runpath = path;
                while (comprun) {
                    int i = 0;
                    while (i < pafs.length) {
                        boolean found = false;
                        int k = 0;
                        while (!found & (k < 4)) {
                            if (pafs[i][k] == runpath) found = true;
                            else k++;
                        }
                        if (found) {
                            if ((crossings[i] % 2) == 0) {
                                if (k == 0) pos = 1;
                                if (k == 1) pos = 0;
                                if (k == 2) pos = 3;
                                if (k == 3) pos = 2;
                            }
                            else {
                                if (k == 0) pos = 2;
                                if (k == 1) pos = 3;
                                if (k == 2) pos = 0;
                                if (k == 3) pos = 1;
                            }
                            endpath = pafs[i][pos];
                            pafs[i][k] = 0;
                            pafs[i][pos] = 0;
                            if (endpath != path) {
                                notcheckedpaths[endpath - 1] = false;
                                comp.add(endpath);
                                runpath = endpath;
                            }
                            else comprun = false;
                            break;
                        }
                        i++;
                    }
                }
                compts.add(comp);
                int j = 1;
                while (j < 2 * crossings.length) {
                    if (notcheckedpaths[j]) break;
                    j++;
                }
                path = j+1;
            }
            return compts;
        }
        
    }
    

    public static void main(String[] args) {
    	String [] arg = {"-st", "0", "-6", "3", "-dg", "-1,2.2,3.3,-1." };
        if (args.length > 0) arg = args;
        KnotJob job = new KnotJob(arg);
        
    }

}